 /***************************************************************************
                         callbacks.c  -  description
                             -------------------
    begin                : Sun May 19 2002
    copyright            : (C) 2002 by Krishna Jaidev and N Deepak
    email                : jaidev@symonds.net | deepak@phreaker.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include <gnome.h>
#include <libgnomeui/gnome-color-picker.h>
#include <libgnomeprint/gnome-printer.h>
#include <libgnomeprint/gnome-print.h>

#include <gtkhtml/gtkhtml.h>
#include <gtkhtml/gtkhtml-search.h>
#include <gtkhtml/htmlcolorset.h>
#include <gtkhtml/htmlcolor.h>
#include <gtkhtml/htmlengine.h>
#include <gtkhtml/htmlpainter.h>
#include <gtkhtml/htmlsettings.h>
#include <gtkhtml/htmlimage.h>
#include <gtkhtml/htmltokenizer.h>
#include <gtkhtml/htmlengine-edit-cut-and-paste.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "functions.h"

#define ON 1
#define OFF 0


/* ***************************************************************** */
/* *** Menu callbacks. *** */


/* *** File menu callbacks. *** */

void
on_new_file_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    new_file ();
}

void
on_open_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    get_filename (OPR_OPEN);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern int FILE_OPEN;
    if (FILE_OPEN)
        save_current_view (FALSE);
    else
        get_filename(OPR_SAVE_AS);
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    get_filename (OPR_SAVE_AS);
}


void
on_close1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    new_file ();
}


void
on_exit_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern gchar *TMP_FILE_NAME, *TMP2_FILE_NAME;
    extern int TMP_FILE_DESC;
    if (TMP_FILE_DESC != -1) {
        unlink (TMP_FILE_NAME);
        unlink (TMP2_FILE_NAME);
        printf ("Removed ");
    }
    gtk_main_quit ();
}

/* ***************************************************************** */

/* *** Edit menu callbacks. *** */

void
on_undo1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern int CURRENT_VIEW;
    if (CURRENT_VIEW == 0)
        gtk_html_undo (GTK_HTML(html_edit1));
}


void
on_redo1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern int CURRENT_VIEW;

    if (CURRENT_VIEW == 0)
        gtk_html_redo (GTK_HTML(html_edit1));
}


void
on_select_all1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern GtkWidget* text1;
    extern int CURRENT_VIEW;

    if (CURRENT_VIEW == 0)
        gtk_html_select_all (GTK_HTML(html_edit1));
    else if (CURRENT_VIEW == 1)
        gtk_editable_select_region (GTK_EDITABLE(text1), 0, -1);
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern GtkWidget* text1;
    extern int CURRENT_VIEW;

    if (CURRENT_VIEW == 0)
        gtk_html_cut (GTK_HTML(html_edit1));
    else if (CURRENT_VIEW == 1)
        gtk_editable_cut_clipboard (GTK_EDITABLE(text1));
}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern GtkWidget* text1;
    extern int CURRENT_VIEW;

    if (CURRENT_VIEW == 0)
        gtk_html_copy (GTK_HTML(html_edit1));
    else if (CURRENT_VIEW == 1)
        gtk_editable_copy_clipboard (GTK_EDITABLE(text1));
}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget* html_edit1;
    extern GtkWidget* text1;
    extern int CURRENT_VIEW;

    if (CURRENT_VIEW == 0)
        gtk_html_paste (GTK_HTML(html_edit1));
    else if (CURRENT_VIEW == 1)
        gtk_editable_paste_clipboard (GTK_EDITABLE(text1));
}


void
on_find1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* dialog_box1 = create_search_dialog();
    gtk_widget_show (dialog_box1);
}



void
on_replace1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* dialog_box = create_replace_dialog();
    gtk_widget_show (dialog_box);
}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* prop;
    prop = create_properties_dialog (COLORS);
    gtk_widget_show (prop);
}

/* ***************************************************************** */

/* *** Insert menu callbacks. *** */

void
on_line_break_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_insert_html (GTK_HTML (html_edit1), "<br>&nbsp;");
}

void
on_paragraph_break_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_INSERT_PARAGRAPH");
}


void
on_line1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkHTML* html_edit1;
    gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_INSERT_RULE");
}


void
on_hyperlink1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* link_dlg_box;
    link_dlg_box = create_link_dialog ();
    gtk_widget_show (link_dlg_box);
}


void
on_image1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* img_dlg_box;
    img_dlg_box = create_image_dialog ();
    gtk_widget_show (img_dlg_box);
}


void
on_html_insert_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *html_wid = create_html_insert_dialog ();
    gtk_widget_show(html_wid);
}

/* ***************************************************************** */

/* *** Format menu callbacks. *** */

void on_font_variable_width_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1), 0x7f, 0);
}

void on_font_fixed_width_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1), 0x7f, GTK_HTML_FONT_STYLE_FIXED);
}


void on_font_size_2_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK , GTK_HTML_FONT_STYLE_SIZE_1);

}

void on_font_size_1_activate (GtkWidget *was_clicked, gpointer user_data)
{

    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_2);
}

void on_font_size0_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_3);
}

void on_font_size1_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_4);
}

void on_font_size2_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK , GTK_HTML_FONT_STYLE_SIZE_5);

}

void on_font_size3_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1),   0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_6);
}

void on_font_size4_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_font_style (GTK_HTML(html_edit1), 0xf8 | !GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_7);

}


void
on_bold1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_BOLD_TOGGLE");
}

void
on_italic1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_ITALIC_TOGGLE");

}

void
on_underline1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_UNDERLINE_TOGGLE");

}

void
on_strikeout1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_STRIKEOUT_TOGGLE");
}


void
on_colour1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *font_color_dialog;
    font_color_dialog = create_font_color_dialog ();
    gtk_widget_show (font_color_dialog);
}


void
on_normal1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);

}

void
on_heading_1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H1);
}

void
on_heading_2_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H2);
}

void
on_heading_3_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H3);
}

void
on_heading_4_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H4);
}

void
on_heading_5_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H5);
}

void
on_heading_6_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_H6);
}


void
on_bulletted1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED);
}

void
on_numbered1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT);
}

void on_alphabets1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_ITEMALPHA);
}

void on_roman1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN);
}


void on_para_normal_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
}

void on_para_address_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_ADDRESS)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_ADDRESS);
}

void on_para_formatted_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    if (gtk_html_get_paragraph_style (GTK_HTML(html_edit1)) == GTK_HTML_PARAGRAPH_STYLE_PRE)
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_NORMAL);
    else
        gtk_html_set_paragraph_style (GTK_HTML (html_edit1), GTK_HTML_PARAGRAPH_STYLE_PRE);
}


void
on_left1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_alignment (GTK_HTML(html_edit1), GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT);
}

void
on_centre1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_alignment (GTK_HTML(html_edit1), GTK_HTML_PARAGRAPH_ALIGNMENT_CENTER);

}

void
on_right1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_set_paragraph_alignment (GTK_HTML(html_edit1), GTK_HTML_PARAGRAPH_ALIGNMENT_RIGHT);

}


void on_increase_indent_activate  (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_INDENT_INC");
}

void on_decrease_indent_activate  (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_INDENT_DEC");
}

/* ***************************************************************** */

/* *** Table menu callbacks. */

void
on_new_table1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget* table_dlg_box;
    table_dlg_box = create_table_dialog ();
    gtk_widget_show (table_dlg_box);
}


void
on_delete_table1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_DELETE_TABLE");
}


void
on_insert_before1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_INSERT_ROW_BEFORE");
}

void
on_insert_after1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_INSERT_ROW_AFTER");
}

void
on_delete1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_DELETE_ROW");
}


void
on_insert_before2_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_INSERT_COL_BEFORE");
}

void
on_insert_after2_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_INSERT_COL_AFTER");
}

void
on_delete2_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *html_edit1;
    gtk_html_command (GTK_HTML(html_edit1), "GTK_HTML_COMMAND_TABLE_DELETE_COL");
}

/* ***************************************************************** */

/* *** Help menu callbacks. *** */

void
on_working_with_ginf1_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    system ("konqueror file:/usr/local/doc/ginf/index.html > /dev/null &");
    if (system ("echo $? > /dev/null") != 0)
        create_msg_box ("There was an error opening the help files.\nCheck if they exist in /usr/local/doc/ginf diretory.", "Error opening help files", GNOME_MESSAGE_BOX_ERROR);
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *about_wid = create_about1();
    gtk_widget_show (about_wid);    
}

/* ***************************************************************** */

/* Tab-switch callbacks. */

void
on_preview_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *notebook1;
    gtk_notebook_set_page (GTK_NOTEBOOK(notebook1), 2); 
}

void
on_edit_area_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *notebook1;
    gtk_notebook_set_page (GTK_NOTEBOOK(notebook1), 0); 

}

void
on_source_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    extern GtkWidget *notebook1;
    gtk_notebook_set_page (GTK_NOTEBOOK(notebook1), 1);
}


/* ***************************************************************** */

/* *** Signal handlers for Ginf *** */

void
on_notebook1_switch_page (GtkNotebook *notebook, GtkNotebookPage *page, gint page_num, gpointer data)
{
    extern GtkWidget *html_edit1;
    extern GtkWidget *text1;
    extern GtkWidget *html_preview1;
    extern int TMP_FILE_DESC;
    extern gchar TMP_FILE_NAME[17];
    GtkHTMLStream *stream;
    char buf[4096];
    int bytes;
    extern int CURRENT_VIEW;
    
    // save previous view to temp file
    save_current_view(TRUE);
    
    //gboolean  gtk_html_save  (GtkHTML *html,
    //GtkHTMLSaveReceiverFn receiver, gpointer data);

    // load from temp file to new view
    TMP_FILE_DESC = open (TMP_FILE_NAME, O_RDONLY);
    
    switch (page_num) {
        case 0: // new page is edit
            gtk_html_set_editable (GTK_HTML(html_edit1), FALSE);
            stream = gtk_html_begin(GTK_HTML(html_edit1));
            if (TMP_FILE_DESC == -1) {
                fprintf (stderr, "Can't read file:");
                exit(1);
            }
            do {
                bytes = read(TMP_FILE_DESC, buf, 4096);
                if (bytes > 0)
//                  gtk_html_write(GTK_HTML(html_edit1), stream, buf, strlen(buf));
                    gtk_html_write(GTK_HTML(html_edit1), stream, buf, bytes);
            } while (bytes > 0);
            gtk_html_end(GTK_HTML(html_edit1), stream, GTK_HTML_STREAM_OK);
            gtk_html_set_editable (GTK_HTML(html_edit1), TRUE);
            gtk_widget_grab_focus (html_edit1);
            break;
        case 1: // new page is source
            gtk_editable_delete_text (GTK_EDITABLE (text1), 0, -1);
            if (TMP_FILE_DESC == -1) {
                fprintf (stderr, "Can't read file:");
                exit(1);
            }
            gtk_text_set_point (GTK_TEXT (text1), 0);
            do {
                bytes = read(TMP_FILE_DESC, buf, 4096);
                if (bytes > 0)
                    gtk_text_insert (GTK_TEXT (text1), NULL, NULL, NULL, (char *) buf, bytes);
            } while (bytes > 0);
            gtk_text_set_editable (GTK_TEXT (text1), TRUE);
            gtk_text_set_word_wrap (GTK_TEXT (text1), TRUE);
            gtk_widget_grab_focus (text1);
            break;
        case 2: // new page is preview
            stream = gtk_html_begin(GTK_HTML(html_preview1));
            if (TMP_FILE_DESC == -1) {
                fprintf (stderr, "Can't read file:");
                exit(1);
            }
            do {
                bytes = read(TMP_FILE_DESC, buf, 4096);
                if (bytes > 0)
//                  gtk_html_write(GTK_HTML(html_preview1), stream, buf, strlen(buf));
                    gtk_html_write(GTK_HTML(html_preview1), stream, buf, bytes);
            } while (bytes > 0);
            gtk_html_end(GTK_HTML(html_preview1), stream, GTK_HTML_STREAM_OK);
            gtk_widget_grab_focus (html_preview1);
            break;
    }
    // update CURRENT_VIEW to new view
    close (TMP_FILE_DESC);
    TMP_FILE_DESC = 0;
    CURRENT_VIEW = page_num;
}

/* ***************************************************************** */

/* *** Font tool-bar signal handlers. *** */

void on_font_size_spinbutton_changed (GtkWidget *was_clicked, gpointer user_data)
{
    gint value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (was_clicked));
    switch (value) {
        case -2:
            on_font_size_2_activate (was_clicked, user_data);
            break;
        case -1:
            on_font_size_1_activate (was_clicked, user_data);
            break;
        case 0:
            on_font_size0_activate (was_clicked, user_data);
            break;

        case 1:
            on_font_size1_activate (was_clicked, user_data);
            break;

        case 2:
            on_font_size2_activate (was_clicked, user_data);
            break;

        case 3:
            on_font_size3_activate (was_clicked, user_data);
            break;

        case 4:
            on_font_size4_activate (was_clicked, user_data);
            break;
    }
}

/* ***************************************************************** */

/* *** File-menu file-selection widget signal handlers. *** */

void fs_cancel_callback (GtkWidget *was_clicked)
{
    GtkWidget *file_sel = gtk_widget_get_ancestor (was_clicked, GTK_TYPE_FILE_SELECTION);
    gtk_widget_hide (file_sel);
}

void fs_ok_callback (GtkWidget *was_clicked, gpointer operation)
{
    int fd;
    GtkHTMLStream *stream;
    char buf[4096];
    int bytes;
    extern gchar *OPEN_FILE_NAME;
    extern GtkWidget *html_edit1;
    extern GtkWidget *notebook1;
    extern int FILE_OPEN;

    GtkWidget *file_sel = gtk_widget_get_ancestor (was_clicked, GTK_TYPE_FILE_SELECTION);
    gtk_widget_hide (file_sel);
    OPEN_FILE_NAME = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_sel));
    FILE_OPEN = TRUE;
    if ((int) operation == OPR_OPEN) {
        gtk_notebook_set_page (GTK_NOTEBOOK(notebook1), 0);
        gtk_html_set_editable (GTK_HTML (html_edit1), FALSE);
        stream = gtk_html_begin(GTK_HTML(html_edit1));
        fd = open(OPEN_FILE_NAME, O_RDONLY);
        if (fd == -1) {
            fprintf (stderr, "Can't read file:");
            exit(1);
        }
        do {
            bytes = read(fd, buf, 4096);
            if (bytes > 0) {
                gtk_html_write(GTK_HTML(html_edit1), stream, buf, bytes);
            }
        } while (bytes > 0);
        gtk_html_end(GTK_HTML(html_edit1), stream, GTK_HTML_STREAM_OK);
        close (fd);
        gtk_html_set_editable (GTK_HTML (html_edit1), TRUE);
    }
    else {
        save_current_view (FALSE);
    }   
    if ((int) operation == OPR_SAVE_AS_BEFORE_OPEN)
        get_filename (OPR_OPEN);
    sprintf (buf, "Ginf - %s", OPEN_FILE_NAME);
    gtk_window_set_title     (GTK_WINDOW (gtk_widget_get_toplevel(html_edit1)), buf);
}

/* ***************************************************************** */

/* *** Edit-menu search-dialogue-box signal handlers. *** */

void search_begin_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr;
    search_attr->from_beginning = TRUE;
}

void search_cursor_clicked (GtkWidget *was_clicked, gpointer user_data) {
    extern search_properties *search_attr;
    search_attr->from_beginning = FALSE;
}

void search_fwd_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr;
    search_attr->forward = TRUE;

}

void search_back_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr;
    search_attr->forward = FALSE;
}

void search_case_sens_clicked (GtkWidget *was_clicked, gpointer user_data) {
    extern search_properties *search_attr;
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (was_clicked)) == TRUE)
        search_attr->case_sensitive = TRUE;
    else
        search_attr->case_sensitive = FALSE;
}

void search_regex_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr;
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (was_clicked)) == TRUE)
        search_attr->regular_expression = TRUE;
    else
        search_attr->regular_expression = FALSE;

}

void search_find_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget* html_edit1;
    extern search_properties *search_attr;
    gboolean result;
    gchar *text = gtk_editable_get_chars (GTK_EDITABLE (user_data), 0, -1);
    if (search_attr->from_beginning == TRUE) {
        if (search_attr->forward == TRUE)
            gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_CURSOR_BOD");
        else
            gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_CURSOR_EOD");
    }
    result = gtk_html_engine_search (GTK_HTML(html_edit1), text, search_attr->case_sensitive, search_attr->forward,search_attr->regular_expression);
    if (result != TRUE) {
        create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
    }
    else search_attr->done_find = TRUE;
}

void search_next_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget* html_edit1;
    extern search_properties *search_attr;
    if (search_attr->done_find == TRUE) {
        gtk_html_engine_search_set_forward  (GTK_HTML(html_edit1), !search_attr->forward);
        if (gtk_html_engine_search_next (GTK_HTML (html_edit1)) == FALSE)
            create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
    }
    else search_find_clicked (was_clicked, user_data);  
        
}

void search_prev_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget* html_edit1;
    extern search_properties *search_attr;
    if (search_attr->done_find == TRUE) {
        gtk_html_engine_search_set_forward  (GTK_HTML(html_edit1), search_attr->forward);
        if (gtk_html_engine_search_next (GTK_HTML (html_edit1)) == FALSE)
            create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
    }
    else search_find_clicked (was_clicked, user_data);
}

void search_close_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr;
    search_attr->done_find = FALSE;
    gtk_widget_hide (gtk_widget_get_toplevel(was_clicked));
    g_free (search_attr);
}

/* ***************************************************************** */

/* *** Edit-menu replace-dialogue-box signal-handlers. *** */

void reset_done_find (GtkWidget *was_clicked, gpointer user_data)
{
    extern search_properties *search_attr, *replace_attr;
    int opr = (int) user_data;
    if (opr == OPR_FIND)
        search_attr->done_find = FALSE; 
    else if (opr == OPR_REPLACE)
        replace_attr->done_find = FALSE;
}

void replace_dialog_cb (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    extern search_properties *replace_attr;
    gchar *st, *rt, msgstr[200];
    gboolean result;
    int num = 0, opr = (int) user_data;
    st = gtk_editable_get_chars (GTK_EDITABLE (replace_attr->w1), 0, -1);
    rt = gtk_editable_get_chars (GTK_EDITABLE (replace_attr->w2), 0, -1);
    if (opr == OPR_FIND) {
        if (replace_attr->done_find == FALSE) {
            gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_CURSOR_BOD");
            replace_attr->done_find = TRUE;
        }
        result = gtk_html_engine_search (GTK_HTML(html_edit1), st, FALSE, TRUE, FALSE);
        if (result != TRUE)
            create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
    }
    else if (opr == OPR_REPLACE && replace_attr->done_find == TRUE) {
        gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_DELETE");
        gtk_html_insert_html (GTK_HTML (html_edit1), rt);
        result = gtk_html_engine_search (GTK_HTML(html_edit1), st, FALSE, TRUE, FALSE);
        if (result != TRUE)
            create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
    }
    else if (opr == OPR_REPLACE_ALL) {
        gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_CURSOR_BOD");
        result = FALSE;
        num = 0;
        do {
            result = gtk_html_engine_search (GTK_HTML(html_edit1), st, FALSE, TRUE, FALSE);
            if (result == TRUE) {
                gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_DELETE");
                gtk_html_insert_html (GTK_HTML (html_edit1), rt);
                num ++;
            }
        } while (result == TRUE);
        if (num == 0)
            create_msg_box ("Text not found or end of document reached.", "Failed", GNOME_MESSAGE_BOX_ERROR);
        else {
            sprintf (msgstr, "Found and replaced %d occurance(s).", num);
            create_msg_box (msgstr,"Success",GNOME_MESSAGE_BOX_INFO);
        }
    }
    if (opr == OPR_CLOSE) {
        g_free (replace_attr);
        gtk_widget_hide (gtk_widget_get_toplevel (was_clicked));
    }
}

/* ***************************************************************** */

/* *** Edit-menu page-properties dialogue box signal handlers. *** */

void on_properties_ok_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern widgets *properties_widget_set;
    extern widgets *meta_widget_set;
    extern GtkWidget *notebook1;
    extern ginf_properties DOC_PROPERTIES;
    extern GtkWidget *html_edit1;
    int operation = (int) user_data;
    
    gboolean state;
    guint8 color[4];
    guint16 color16[4];
    GdkColor *color_gdk;
    HTMLImagePointer *img_pointer;
    gchar  *input;
    GtkHTML *html = GTK_HTML (html_edit1);
    
    color_gdk = g_malloc (sizeof (GdkColor));
    if (operation == OPR_OK) {
        gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
        g_free (meta_widget_set);
    }
    
    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (properties_widget_set->w1));
    
    if (state == TRUE) {
        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (properties_widget_set->w2), &color[0], &color[1], &color[2], &color[3]);
        DOC_PROPERTIES.text = color[0];
        DOC_PROPERTIES.text = (DOC_PROPERTIES.text << 8)| color[1];
        DOC_PROPERTIES.text = (DOC_PROPERTIES.text << 8)| color[2];
        gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (properties_widget_set->w2), &color16[0], &color16[1], &color16[2], &color16[3]);
        color_gdk->red = color16[0];
        color_gdk->green = color16[1];
        color_gdk->blue = color16[2];
        html_colorset_set_color (html->engine->settings->color_set, color_gdk, HTMLTextColor);
        
        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (properties_widget_set->w3), &color[0], &color[1], &color[2], &color[3]);
        DOC_PROPERTIES.link = color[0];
        DOC_PROPERTIES.link = (DOC_PROPERTIES.link << 8)| color[1];
        DOC_PROPERTIES.link = (DOC_PROPERTIES.link << 8)| color[2];
        
        gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (properties_widget_set->w3), &color16[0], &color16[1], &color16[2], &color16[3]);
        color_gdk->red = color16[0];
        color_gdk->green = color16[1];
        color_gdk->blue = color16[2];
        html_colorset_set_color (html->engine->settings->color_set, color_gdk, HTMLLinkColor);
        
        
        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (properties_widget_set->w4), &color[0], &color[1], &color[2], &color[3]);
        DOC_PROPERTIES.alink = color[0];
        DOC_PROPERTIES.alink = (DOC_PROPERTIES.alink << 8)| color[1];
        DOC_PROPERTIES.alink = (DOC_PROPERTIES.alink << 8)| color[2];

        gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (properties_widget_set->w4), &color16[0], &color16[1], &color16[2], &color16[3]);
        color_gdk->red = color16[0];
        color_gdk->green = color16[1];
        color_gdk->blue = color16[2];
        html_colorset_set_color (html->engine->settings->color_set, color_gdk, HTMLALinkColor);
        
        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (properties_widget_set->w5), &color[0], &color[1], &color[2], &color[3]);
        DOC_PROPERTIES.vlink = color[0];
        DOC_PROPERTIES.vlink = (DOC_PROPERTIES.vlink << 8)| color[1];
        DOC_PROPERTIES.vlink = (DOC_PROPERTIES.vlink << 8)| color[2];

        gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (properties_widget_set->w5), &color16[0], &color16[1], &color16[2], &color16[3]);
        color_gdk->red = color16[0];
        color_gdk->green = color16[1];
        color_gdk->blue = color16[2];
        html_colorset_set_color (html->engine->settings->color_set, color_gdk, HTMLVLinkColor);

        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (properties_widget_set->w6), &color[0], &color[1], &color[2], &color[3]);
        DOC_PROPERTIES.bgcolor = color[0];
        DOC_PROPERTIES.bgcolor = (DOC_PROPERTIES.bgcolor << 8)| color[1];
        DOC_PROPERTIES.bgcolor = (DOC_PROPERTIES.bgcolor << 8)| color[2];
        
        gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (properties_widget_set->w6), &color16[0], &color16[1], &color16[2], &color16[3]);
        color_gdk->red = color16[0];
        color_gdk->green = color16[1];
        color_gdk->blue = color16[2];
        html_colorset_set_color (html->engine->settings->color_set, color_gdk, HTMLBgColor);

    }
    gtk_html_update_styles (GTK_HTML (html_edit1));
    g_free (color_gdk);
    state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (properties_widget_set->w7));
    if (state == TRUE) {
        img_pointer = g_new (HTMLImagePointer, 1);
        img_pointer->refcount = 1;
        img_pointer->url = g_strdup (gtk_editable_get_chars (GTK_EDITABLE (properties_widget_set->w8), 0, -1));
        img_pointer->loader = gdk_pixbuf_loader_new ();
        img_pointer->pixbuf = NULL;
        img_pointer->animation = NULL;
        img_pointer->interests = NULL;
        img_pointer->factory = html->engine->image_factory;
        img_pointer->stall = FALSE;
        img_pointer->stall_timeout = 0;
        
        html->engine->bgPixmapPtr = img_pointer;
    }
    

    gtk_notebook_set_page (GTK_NOTEBOOK (notebook1),1);
    gtk_notebook_set_page (GTK_NOTEBOOK (notebook1),0);
    
    input = gtk_editable_get_chars (GTK_EDITABLE (properties_widget_set->w9), 0, -1);
    if (input != NULL) {
        if (html->engine->title != NULL)
            g_string_free (html->engine->title, TRUE);
        html->engine->title = g_string_new (input);
        DOC_PROPERTIES.title = input;
    }
    if (strlen (input = gtk_editable_get_chars (GTK_EDITABLE (properties_widget_set->w10),0, -1)) != 0) {
        if (DOC_PROPERTIES.author != NULL)
            g_free (DOC_PROPERTIES.author);
        DOC_PROPERTIES.author = input;
    }
    if (strlen (input = gtk_editable_get_chars (GTK_EDITABLE(properties_widget_set->w11), 0, -1)) != 0) {
        if (DOC_PROPERTIES.description != NULL)
            g_free (DOC_PROPERTIES.description);
        DOC_PROPERTIES.description = input;
    }
    if (strlen(input =gtk_editable_get_chars (GTK_EDITABLE (properties_widget_set->w12), 0 , -1)) != 0) {
        if (DOC_PROPERTIES.keywords != NULL)
            g_free (DOC_PROPERTIES.keywords);   
        DOC_PROPERTIES.keywords = input;
    }
    if (strlen(input =gtk_editable_get_chars (GTK_EDITABLE (properties_widget_set->w13), 0, -1)) != 0) {
        if (DOC_PROPERTIES.classification != NULL)
            g_free (DOC_PROPERTIES.classification);
        DOC_PROPERTIES.classification = input;
    }   
    if (operation == OPR_OK)
        g_free (properties_widget_set);
    
}

void on_properties_cancel_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern widgets *meta_widget_set;
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
    g_free (meta_widget_set);
    
}


void on_dibox_browse_activate (GtkWidget *was_clicked, gpointer user_data)
{
    GtkWidget *file_selection;
    file_selection = gtk_file_selection_new ("Browse...");
    gtk_signal_connect (GTK_OBJECT ( GTK_FILE_SELECTION (file_selection)->ok_button), "clicked", on_dibox_browse_ok_callback, user_data);
    gtk_signal_connect (GTK_OBJECT ( GTK_FILE_SELECTION (file_selection)->cancel_button), "clicked", on_dibox_browse_cancel_callback, NULL);
    gtk_widget_show (file_selection);   
}

void on_dibox_browse_ok_callback (GtkWidget *was_clicked, gpointer user_data)
{
    GtkWidget *file_sel;
    GtkWidget *text_entry = (GtkWidget *) user_data;
    file_sel = gtk_widget_get_toplevel (was_clicked);
    gtk_widget_hide (file_sel);
    gtk_entry_set_text (GTK_ENTRY (text_entry), gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel)));
}

void on_dibox_browse_cancel_callback (GtkWidget *was_clicked, gpointer user_data)
{
    gtk_widget_hide (gtk_widget_get_toplevel (was_clicked));    
}


void  on_meta_tags_clear_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern widgets *meta_widget_set;
    gtk_entry_set_text (GTK_ENTRY (meta_widget_set->w1), "");
    gtk_entry_set_text (GTK_ENTRY (meta_widget_set->w2), "");

}

void  on_meta_tags_add_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern widgets *meta_widget_set;
    
    gchar *name, *content;
    name = gtk_editable_get_chars (GTK_EDITABLE (meta_widget_set->w1), 0, -1);
    content = gtk_editable_get_chars (GTK_EDITABLE (meta_widget_set->w2), 0, -1);
    if (name != NULL && content != NULL) {
        ginf_meta_add (name, content);
        gtk_entry_set_text (GTK_ENTRY (meta_widget_set->w1), " ");
        gtk_entry_set_text (GTK_ENTRY (meta_widget_set->w2), " ");
    }
}

/* ***************************************************************** */

/* *** Insert-menu link-dialogue-box signal handlers. *** */

void link_opts_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    extern link_properties *link_props;
    link_props->type = (int) user_data;
}

void insert_link_cb (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    extern link_properties *link_props;
    gchar *text, *url, *code, type[9];
    int textlen, urllen, opr = (int) user_data, typelen;
    if (opr == OPR_OK || opr == OPR_APPLY) {
        gtk_html_command (GTK_HTML (html_edit1), "GTK_HTML_COMMAND_DELETE");
        switch (link_props->type) {
            case 0:
                strcpy (type, "http://");
                break;
            case 1:
                strcpy (type, "https://");
                break;
            case 2:
                strcpy (type, "ftp://");
                break;
            case 3:
                strcpy (type, "gopher://");
                break;
            case 4:
                strcpy (type, "telnet://");
                break;
            case 5:
                strcpy (type, "mailto:");
                break;
            case 6:
                strcpy (type, "news:");
                break;
            case 7:
                strcpy (type, "");
                break;
        }
        text = gtk_editable_get_chars (GTK_EDITABLE (link_props->w1), 0, -1);
        url = gtk_editable_get_chars (GTK_EDITABLE (link_props->w2), 0, -1);
        textlen = strlen (text);
        urllen = strlen (url);
        typelen = strlen (type);
        code = g_malloc ((14 + textlen + urllen + typelen) * sizeof (char));
        sprintf (code, "<A HREF=\"%s%s\">%s</A>", type, url, text);
        gtk_html_insert_html (GTK_HTML (html_edit1), code);
    }
    if (opr == OPR_OK || opr == OPR_CANCEL) {
        // ok or cancel
        gtk_widget_hide (gtk_widget_get_toplevel(was_clicked));
        g_free (link_props);
    
    }
    
    
}

/* ***************************************************************** */

/* *** Insert-menu image-dialogue-box signal handlers. *** */

void on_image_ok_activate (GtkWidget *was_clicked, gpointer user_data)
{
    gchar buf[5120];
    gchar alt_buf[257];
    gchar tmp_buf[512];
    int hz_spc, vrt_spc;
    int border;
    int height, width;

    extern widgets *img_widget_set;
    extern image_properties *img_props_set;
    extern GtkWidget* html_edit1;

    hz_spc = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(img_widget_set->w2));
    vrt_spc = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(img_widget_set->w3));
    border = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(img_widget_set->w5));
    height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(img_widget_set->w6));
    width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(img_widget_set->w7));
    strcpy (alt_buf, gtk_entry_get_text (GTK_ENTRY(img_widget_set->w4)));

    sprintf (buf, "<IMG ");
    sprintf (tmp_buf, " SRC=\"%s\" ", gtk_entry_get_text (GTK_ENTRY(img_widget_set->w1)));
    strcat (buf, tmp_buf);

    if (img_props_set->align == 0)
        strcat (buf, "ALIGN=LEFT ");
    else if (img_props_set->align == 1)
        strcat (buf, "ALIGN=MIDDLE ");
    else
        strcat (buf, "ALIGN=RIGHT ");

    if (img_props_set->has_cust_size == TRUE) {
        sprintf (tmp_buf, "WIDTH=%u", width);

        if (img_props_set->width_in_percent == TRUE)
            strcat (tmp_buf, "% ");
        else
            strcat (tmp_buf, " ");
        strcat (buf, tmp_buf);

        sprintf (tmp_buf, "HEIGHT=%u", width);

        if (img_props_set->height_in_percent == TRUE)
            strcat (tmp_buf, "% ");
        else
            strcat (tmp_buf, " ");
        strcat (buf, tmp_buf);
    }

    if (img_props_set->has_border == TRUE)
        sprintf (tmp_buf, " BORDER=%u ", border);
    else
        sprintf (tmp_buf, " BORDER=0 ");
    strcat (buf, tmp_buf);

    sprintf (tmp_buf, " VSPACE=%d HSPACE=%d ", vrt_spc, hz_spc);
    strcat (buf, tmp_buf);

    if (strlen (alt_buf) != 0) {
        sprintf (tmp_buf, " ALT=\"%s\"", alt_buf);
        strcat (buf, tmp_buf);
    }

    strcat (buf, ">");
    gtk_html_insert_html (GTK_HTML (html_edit1), buf);

    g_free (img_widget_set);
    g_free (img_props_set);
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}

void on_image_apply_activate (GtkWidget *was_clicked, gpointer user_data)
{
    on_image_ok_activate (was_clicked, user_data);
    gtk_widget_show (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}

void on_image_cancel_activate (GtkWidget *was_clicked, gpointer user_data)
{
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}


void img_border_toggle (GtkWidget *was_clicked, gpointer user_data)
{
    extern image_properties *img_props_set;
    if (img_props_set->has_border == FALSE)
        img_props_set->has_border = TRUE;
    else
        img_props_set->has_border = FALSE;
}

void img_custsize_toggle (GtkWidget *was_clicked, gpointer user_data)
{
    extern image_properties *img_props_set;
    if (img_props_set->has_cust_size == FALSE)
        img_props_set->has_cust_size = TRUE;
    else
        img_props_set->has_cust_size = FALSE;
}

/* ***************************************************************** */

/* *** Insert-menu HTML-code signal handlers. *** */

void on_html_ok (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    GtkWidget *html_text = (GtkWidget *) user_data;
    gtk_widget_hide (gtk_widget_get_toplevel (was_clicked));
    gtk_html_insert_html (GTK_HTML (html_edit1), gtk_editable_get_chars (GTK_EDITABLE (html_text), 0, -1));
    
}

void on_html_apply (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget * html_edit1;
    GtkWidget *html_text = (GtkWidget *) user_data;
    gtk_html_insert_html (GTK_HTML (html_edit1), gtk_editable_get_chars (GTK_EDITABLE (html_text), 0, -1));
}

void on_html_cancel (GtkWidget *was_clicked, gpointer user_data)
{
    gtk_widget_hide (gtk_widget_get_toplevel (was_clicked));
}

/* ***************************************************************** */

/* *** Format-menu font-colour-dialogue-box signal handlers. *** */

void on_font_color_ok_activate (GtkWidget *was_clicked, gpointer user_data)
{
    extern GtkWidget *html_edit1;
    GtkWidget *picker;
    gint8 selection[4];
    
    GtkWidget *dialog = gtk_widget_get_toplevel (was_clicked);
    gtk_widget_hide (dialog);
    
    picker = (GtkWidget *) user_data;
    gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (picker), &selection[0], &selection[1], &selection[2], &selection[3]);
    gtk_html_set_color (GTK_HTML (html_edit1), html_color_new_from_rgb (selection[0], selection[1], selection[2]));
    
}

void on_font_color_cancel_activate (GtkWidget *was_clicked, gpointer user_data)
{
    GtkWidget *dialog = gtk_widget_get_toplevel (was_clicked);
    gtk_widget_hide (dialog);
}

/* ***************************************************************** */

/* *** Table-menu table-dialogue-box signal handlers. *** */

void on_table_ok_activate (GtkWidget *was_clicked, gpointer user_data)
{
    gint nrows, ncols;
    gint border_width;
    gint cl_spacing;
    gint cl_padding;
    gint tbl_width;
    gchar buf[5120];
    gchar tmp_buf[512];
    gchar clr_buf[3][3];
    guint8 color8[4];
    int i, j;

    extern widgets *tbl_widget_set;
    extern table_properties *tbl_props_set;
    extern GtkWidget* html_edit1;

    nrows = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w1));
    ncols = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w2));
    border_width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w3));
    cl_spacing = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w4));
    cl_padding = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w5));
    tbl_width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(tbl_widget_set->w6));

    sprintf (buf, "<DIV ");
    if (tbl_props_set->align == 0)
        strcat (buf, "ALIGN=LEFT>");
    else if (tbl_props_set->align == 1)
        strcat (buf, "ALIGN=CENTER>");
    else
        strcat (buf, "ALIGN=RIGHT>");

    strcat (buf, "<TABLE ");
    if (tbl_props_set->has_border == TRUE)
        sprintf (tmp_buf, "BORDER=%u ", border_width);
    else
        sprintf (tmp_buf, "BORDER=0 ");
    strcat (buf, tmp_buf);

    if (tbl_props_set->has_bgnd == TRUE && tbl_props_set->bgnd_is_clr == FALSE) {
        sprintf (tmp_buf, "BACKGROUND=\"%s\"",
            gtk_entry_get_text (GTK_ENTRY(tbl_widget_set->w7)));
        strcat (buf, tmp_buf);
    }

    if (tbl_props_set->has_bgnd == TRUE && tbl_props_set->bgnd_is_clr == TRUE) {
        gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (tbl_widget_set->w8),
            &color8[0], &color8[1], &color8[2], &color8[3]);
        for (i=0; i<3; i++)
            sprintf (clr_buf[i], "%X", color8[i]);

        sprintf (tmp_buf, "BGCOLOR=\"#%s", clr_buf[0]);
        strcat (tmp_buf, clr_buf[1]);
        strcat (tmp_buf, clr_buf[2]);
        strcat (tmp_buf, "\" ");
        printf ("%s\n", tmp_buf);
        strcat (buf, tmp_buf);
    }

    sprintf (tmp_buf, "WIDTH=%u", tbl_width);

    if (tbl_props_set->width_in_percent == TRUE)
        strcat (tmp_buf, "% ");
    else
        strcat (tmp_buf, " ");
    strcat (buf, tmp_buf);

    sprintf (tmp_buf, "CELLSPACING=%u ", cl_spacing);
    strcat (buf, tmp_buf);

    sprintf (tmp_buf, "CELLPADDING=%u ", cl_padding);
    strcat (buf, tmp_buf);

    strcat (buf, "> ");

    for (i=0; i<nrows; i++) {
        strcat (buf, "<TR>");
        for (j=0; j<ncols; j++) {
            strcat (buf, "<TD></TD>");
        }
        strcat (buf, "</TR>");
    }

    strcat (buf, "</TABLE></DIV>");
    gtk_html_insert_html (GTK_HTML (html_edit1), buf);

    g_free (tbl_widget_set);
    g_free (tbl_props_set);
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}

void on_table_apply_activate (GtkWidget *was_clicked, gpointer user_data)
{
    on_table_ok_activate (was_clicked, user_data);
    gtk_widget_show (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}

void on_table_cancel_activate (GtkWidget *was_clicked, gpointer user_data)
{
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}


void table_left_align_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    tbl_props_set->align = 0;
}

void table_center_align_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    tbl_props_set->align = 1;
}

void table_right_align_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    tbl_props_set->align = 2;
}


void table_border_toggle (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    if (tbl_props_set->has_border == FALSE)
        tbl_props_set->has_border = TRUE;
    else
        tbl_props_set->has_border = FALSE;
}

void table_width_percent_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    GtkAdjustment *new_domain = gtk_adjustment_new (100, 1, 100, 1, 10, 10);

    tbl_props_set->width_in_percent = (gboolean) TRUE;
    gtk_spin_button_set_adjustment (user_data, new_domain);
    gtk_spin_button_update (user_data);
}

void table_width_pixel_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    GtkAdjustment *new_domain = gtk_adjustment_new (200, 1, 1000, 1, 10, 10);

    tbl_props_set->width_in_percent = FALSE;
    gtk_spin_button_set_adjustment (user_data, new_domain);
    gtk_spin_button_update (user_data);
}


void table_bgnd_toggle (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    if (tbl_props_set->has_bgnd == FALSE)
        tbl_props_set->has_bgnd = TRUE;
    else
        tbl_props_set->has_bgnd = FALSE;
}

void table_bgcolor_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    tbl_props_set->bgnd_is_clr = TRUE;
}

void table_bgimage_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    extern table_properties *tbl_props_set;
    tbl_props_set->bgnd_is_clr = FALSE;
}

void table_bgimage_browse_clicked (GtkWidget *was_clicked, gpointer user_data)
{
    GtkWidget *file_sel;

    file_sel = gtk_file_selection_new ("Insert background image.. ");
    gtk_signal_connect (GTK_OBJECT ( GTK_FILE_SELECTION (file_sel)->ok_button),
        "clicked", on_bgimage_browse_ok_activate, user_data);
    gtk_signal_connect (GTK_OBJECT ( GTK_FILE_SELECTION (file_sel)->cancel_button),
        "clicked", on_bgimage_browse_cancel_activate, NULL);
    gtk_widget_show (file_sel);
}

void on_bgimage_browse_ok_activate (GtkWidget *was_clicked, gpointer user_data)
{
    GtkWidget *file_sel;

    gtk_widget_hide (file_sel = gtk_widget_get_toplevel (was_clicked));
    gtk_entry_set_text (GTK_ENTRY (user_data),
        gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel)));
}

void on_bgimage_browse_cancel_activate (GtkWidget *was_clicked, gpointer user_data)
{
    gtk_widget_hide (GTK_WIDGET (gtk_widget_get_toplevel (was_clicked)));
}


