/***************************************************************************
                          interface.c  -  description
                             -------------------
    begin                : Sun May 19 2002
    copyright            : (C) 2002 by Krishna Jaidev and N Deepak
    email                : jaidev@symonds.net | deepak@phreaker.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include <gnome.h>
#include <gtkhtml/gtkhtml.h>
#include <stdlib.h>
#include <fcntl.h>
#include <gtkhtml/htmlengine.h>
#include <gtkhtml/htmlselection.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "functions.h"

/* ***************************************************************** */
/* *** Menu-UI information. *** */

/* *** File menu. *** */

static GnomeUIInfo file1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New File"),
        N_("Create a new HTML document"), on_new_file_activate, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (on_open_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_SAVE_ITEM (on_save1_activate, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (on_save_as1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CLOSE_ITEM (on_close1_activate, NULL),
  GNOMEUIINFO_MENU_EXIT_ITEM (on_exit_activate, NULL),
  GNOMEUIINFO_END
};

/* *** Edit menu. *** */
static GnomeUIInfo edit1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_UNDO_ITEM (on_undo1_activate, NULL),
  GNOMEUIINFO_MENU_REDO_ITEM (on_redo1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Select _All"),
    N_("Select the entire document"),
    on_select_all1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    GDK_a, GDK_CONTROL_MASK, NULL
  },
  GNOMEUIINFO_MENU_CUT_ITEM (on_cut1_activate, NULL),
  GNOMEUIINFO_MENU_COPY_ITEM (on_copy1_activate, NULL),
  GNOMEUIINFO_MENU_PASTE_ITEM (on_paste1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_FIND_ITEM (on_find1_activate, NULL),
  GNOMEUIINFO_MENU_REPLACE_ITEM (on_replace1_activate, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PROPERTIES_ITEM (on_properties1_activate, NULL),
  GNOMEUIINFO_END
};

/* *** Insert menu. *** */

static GnomeUIInfo break1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  _Line"),
    N_("Start a new line"),
    on_line_break_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Paragraph  "),
    N_("Start a new paragraph"),
    on_paragraph_break_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo insert1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_SUBTREE, N_("_Break"),
    N_("Break the text"),
    break1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Hori_zontal Line"),
    N_("Insert a horizontal line"),
    on_line1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "ginf/tb_separator.xpm",
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Hyperlink..."),
    N_("Insert a link to another document"),
    on_hyperlink1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "ginf/tb_link.xpm",
    GDK_l, GDK_CONTROL_MASK, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Image..."),
    N_("Insert a picture"),
    on_image1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "ginf/tb_image.xpm",
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_HTML Code..."),
    N_("Insert raw HTML"),
    on_html_insert_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },

  GNOMEUIINFO_END
};

/* *** Format menu. *** */

static GnomeUIInfo font_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  _Variable Width  "),
    NULL,
    on_font_variable_width_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Fixed Width"),
    NULL,
    on_font_fixed_width_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE,NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo font_size_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  -2  "),
    NULL,
    on_font_size_2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  -1"),
    NULL,
    on_font_size_1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("   _0"),
    NULL,
    on_font_size0_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  +_1"),
    NULL,
    on_font_size1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  +_2"),
    NULL,
    on_font_size2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  +_3"),
    NULL,
    on_font_size3_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  +_4"),
    NULL,
    on_font_size4_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo style1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("_Bold"),
    NULL,
    on_bold1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TEXT_BOLD,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Italic"),
    NULL,
    on_italic1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TEXT_ITALIC,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Underline"),
    NULL,
    on_underline1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TEXT_UNDERLINE,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Strikeout"),
    NULL,
    on_strikeout1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TEXT_STRIKEOUT,
    0, 0, NULL
  },

  GNOMEUIINFO_END
};

static GnomeUIInfo text_type1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  _Normal"),
    NULL,
    on_normal1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _1  "),
    NULL,
    on_heading_1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _2"),
    NULL,
    on_heading_2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _3"),
    NULL,
    on_heading_3_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _4"),
    NULL,
    on_heading_4_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _5"),
    NULL,
    on_heading_5_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Heading _6"),
    NULL,
    on_heading_6_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo list1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  _Bulletted  "),
    NULL,
    on_bulletted1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "ginf/tb_unordered.xpm",
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Numbered"),
    NULL,
    on_numbered1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "ginf/tb_ordered.xpm",
    0, 0, NULL
  },
 {
    GNOME_APP_UI_ITEM, N_("  _Alphabets"),
    NULL,
    on_alphabets1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Roman"),
    NULL,
    on_roman1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo paragraph_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  _Normal"),
    NULL,
    on_para_normal_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Address"),
    NULL,
    on_para_address_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE,NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  _Pre Formatted  "),
    NULL,
    on_para_formatted_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE,NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo align1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("_Left"),
    NULL,
    on_left1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ALIGN_LEFT,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Centre"),
    NULL,
    on_centre1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ALIGN_CENTER,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Right"),
    NULL,
    on_right1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ALIGN_RIGHT,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo format1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_SUBTREE, N_("_Font"),
    NULL,
    font_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FONT,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Font Si_ze"),
    NULL,
    font_size_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Font _Style"),
    NULL,
    style1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Font _Colour"),
    NULL,
    on_colour1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_SUBTREE, N_("_Text Type"),
    NULL,
    text_type1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("_List"),
    NULL,
    list1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("_Paragraph"),
    NULL,
    paragraph_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("_Align"),
    NULL,
    align1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("_Increase Indent"),
    NULL,
    on_increase_indent_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Decrease Indent     "),
    NULL,
    on_decrease_indent_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

/* *** Table menu. *** */

static GnomeUIInfo row1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  Insert Before          "),
    N_("Insert a new row before the current row"),
    (gpointer) on_insert_before1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Insert After"),
    N_("Insert a new row after the current row"),
    (gpointer) on_insert_after1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Delete"),
    N_("Delete the current row"),
    (gpointer) on_delete1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo column1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  Insert Before          "),
    N_("Insert a new column before the current column"),
    (gpointer) on_insert_before2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Insert After"),
    N_("Insert a new column after the current column"),
    (gpointer) on_insert_after2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Delete"),
    N_("Delete the current column"),
    (gpointer) on_delete2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo table1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("  New Table...            "),
    N_("Insert a new table"),
    (gpointer) on_new_table1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("  Delete Table"),
    N_("Delete the current table"),
    (gpointer) on_delete_table1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_SUBTREE, N_("  Row"),
    NULL,
    row1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("  Column"),
    NULL,
    column1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

/* *** Help menu. *** */

static GnomeUIInfo help1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Working with _Ginf..."),
    NULL,
    on_working_with_ginf1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
    GDK_F1, 0, NULL
  },
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_about1_activate, NULL),
  GNOMEUIINFO_END
};

/* *** Menu-bar. *** */

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_EDIT_TREE (edit1_menu_uiinfo),
  {
    GNOME_APP_UI_SUBTREE, N_("_Insert"),
    N_("Insert HTML elements into the document"),
    insert1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Fo_rmat"),
    N_("Format the HTML elements"),
    format1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("_Table"),
    N_("Work with tables"),
    table1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, 0, NULL
  },
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};

/* ***************************************************************** */

/* *** GINF interface creation function. *** */

GtkWidget*
create_app1 (void)
{
  extern GtkWidget *html_edit1, *text1, *html_preview1, *notebook1;

  GtkWidget *app1;
  GtkWidget *dock1;
  GtkWidget *toolbar1;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;
  GtkWidget *button5;
  GtkWidget *button6;
  GtkWidget *button7;
  GtkWidget *button8;
  GtkWidget *button9;
  GtkWidget *toolbar3;
  GtkWidget *Fonts;
  GtkWidget *Fonts_menu;
  GtkWidget *glade_menuitem;
  GtkObject *spinbutton1_adj;
  GtkWidget *spinbutton1;
  GtkWidget *toolbar2;
  GtkWidget *button12;
  GtkWidget *button13;
  GtkWidget *button14;
  GtkWidget *button15;
  GtkWidget *button16;
  GtkWidget *button17;
  GtkWidget *button18;
  GtkWidget *button19;
  GtkWidget *button20;
  GtkWidget *button21;
  GtkWidget *button22;
  GtkWidget *button26;
  GtkWidget *button27;
  GtkWidget *button28;
  GtkWidget *button29;
  GtkWidget *button33;
  GtkWidget *scrolledwindow1;
  GtkWidget *viewport1;
  GtkWidget *scrolledwindow4;
  GtkWidget *label1;
  GtkWidget *scrolledwindow2;
  GtkWidget *viewport2;
  GtkWidget *scrolledwindow5;
  GtkWidget *label2;
  GtkWidget *scrolledwindow3;
  GtkWidget *viewport3;
  GtkWidget *scrolledwindow6;
  GtkWidget *label3;
  GtkWidget *appbar1;
  GtkWidget *optionmenu2;
  GtkWidget *optionmenu2_menu;
        
  app1 = gnome_app_new ("Ginf", _("Ginf"));
  gtk_object_set_data (GTK_OBJECT (app1), "app1", app1);
  gtk_window_set_position (GTK_WINDOW (app1), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (app1), 700, 450);

  dock1 = GNOME_APP (app1)->dock;
  gtk_widget_ref (dock1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "dock1", dock1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dock1);

  /* *** Create the menus. *** */

  gnome_app_create_menus (GNOME_APP (app1), menubar1_uiinfo);

  gtk_widget_ref (menubar1_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "file1",
                            menubar1_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "new_file1",
                            file1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "open1",
                            file1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "save1",
                            file1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "save_as1",
                            file1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "print1",
                            file1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator1",
                            file1_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "close1",
                            file1_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file1_menu_uiinfo[7].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "exit1",
                            file1_menu_uiinfo[7].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "edit1",
                            menubar1_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "undo1",
                            edit1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "redo1",
                            edit1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator5",
                            edit1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "select_all1",
                            edit1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "cut1",
                            edit1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "copy1",
                            edit1_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "paste1",
                            edit1_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[8].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator2",
                            edit1_menu_uiinfo[8].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[9].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "find1",
                            edit1_menu_uiinfo[9].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[10].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator11",
                            edit1_menu_uiinfo[10].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (edit1_menu_uiinfo[11].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "preferences1",
                            edit1_menu_uiinfo[11].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "insert1",
                            menubar1_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "break1",
                            insert1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (break1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "line1",
                            break1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (break1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "paragraph1",
                            break1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "hyperlink1",
                            insert1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "image1",
                            insert1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "horizontal_line1",
                            insert1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "table1",
                            insert1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "meta_tags1",
                            insert1_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "html1",
                            insert1_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (insert1_menu_uiinfo[7].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "form1",
                            insert1_menu_uiinfo[7].widget,
                            (GtkDestroyNotify) gtk_widget_unref);


  gtk_widget_ref (menubar1_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "format1",
                            menubar1_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "font1",
                            format1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "variable1",
                            font_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "fixed1",
                            font_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);


  gtk_widget_ref (format1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "size1",
                            format1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizen2",
                            font_size_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizen1",
                            font_size_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizep0",
                            font_size_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizep1",
                            font_size_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizep2",
                            font_size_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizep3",
                            font_size_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (font_size_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "sizep4",
                            font_size_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);


  gtk_widget_ref (format1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "style1",
                            format1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (style1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "bold1",
                            style1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (style1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "italic1",
                            style1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (style1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "underline1",
                            style1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (style1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "strikeout1",
                            style1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "colour1",
                            format1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator11",
                            format1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "text_type1",
                            format1_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "normal_1",
                            text_type1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_1",
                            text_type1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_2",
                            text_type1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_3",
                            text_type1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_4",
                            text_type1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_5",
                            text_type1_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (text_type1_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "heading_6",
                            text_type1_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "list1",
                            format1_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (list1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "bulletted1",
                            list1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (list1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "numbered1",
                            list1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (list1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "alphabets1",
                            list1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (list1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "roman1",
                            list1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[7].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "align1",
                            format1_menu_uiinfo[8].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (align1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "left1",
                            align1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (align1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "centre1",
                            align1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (align1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "right1",
                            align1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[8].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator13",
                            format1_menu_uiinfo[8].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (format1_menu_uiinfo[9].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "page_properties1",
                            format1_menu_uiinfo[9].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "table1",
                            menubar1_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (table1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "new_table1",
                            table1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (table1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "delete_table1",
                            table1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (table1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "separator3",
                            table1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (table1_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "row1",
                            table1_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (row1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "insert_before1",
                            row1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (row1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "insert_after1",
                            row1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (row1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "delete1",
                            row1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (table1_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "column1",
                            table1_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (column1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "insert_before2",
                            column1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (column1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "insert_after2",
                            column1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (column1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "delete2",
                            column1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "settings1",
                            menubar1_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar1_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "help1",
                            menubar1_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help1_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "working_with_ginf1",
                            help1_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help1_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "report_bug1",
                            help1_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help1_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (app1), "about1",
                            help1_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);


  /* *** Add the main tool-bar. *** */

  toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_ref (toolbar1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "toolbar1", toolbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar1);
  gnome_app_add_toolbar (GNOME_APP (app1), GTK_TOOLBAR (toolbar1), "toolbar1",
                                GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
                                GNOME_DOCK_TOP, 1, 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (toolbar1), 1);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar1), 10);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_SPACE_LINE);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar1), GTK_RELIEF_NONE);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_NEW);
  button1 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("New"),
                                _("New File"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button1);

  gtk_object_set_data_full (GTK_OBJECT (app1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", on_new_file_activate, NULL);
  gtk_widget_show (button1);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_OPEN);
  button2 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Open"),
                                _("Open File"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_signal_connect (GTK_OBJECT (button2),  "clicked", on_open_activate, NULL);
  gtk_widget_show (button2);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_SAVE);
  button3 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save"),
                                _("Save File"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_signal_connect (GTK_OBJECT (button3),  "clicked", on_save1_activate,NULL);
        gtk_widget_show (button3);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_UNDO);
  button8 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Undo"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button8);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button8", button8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button8),  "clicked", on_undo1_activate, NULL);
  gtk_widget_show (button8);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_REDO);
  button9 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Redo"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button9);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button9", button9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button9),  "clicked", on_redo1_activate, NULL);
  gtk_widget_show (button9);
    gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_CUT);
  button4 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Cut"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_signal_connect (GTK_OBJECT (button4),  "clicked", on_cut1_activate, NULL);
  gtk_widget_show (button4);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_COPY);
  button5 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Copy"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button5);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button5", button5,
                            (GtkDestroyNotify) gtk_widget_unref);
gtk_signal_connect (GTK_OBJECT (button5),  "clicked", on_copy1_activate, NULL);
  gtk_widget_show (button5);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_PASTE);
  button6 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Paste"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button6);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button6", button6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button6),  "clicked", on_paste1_activate, NULL);
  gtk_widget_show (button6);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_SEARCH);
  button7 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Search"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button7);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button7", button7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button7),  "clicked", on_find1_activate, NULL);
  gtk_widget_show (button7);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar1));

  tmp_toolbar_icon = create_pixmap (app1, "ginf/tb_link.xpm", TRUE);
  button26 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Link"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button26);
        gtk_signal_connect (GTK_OBJECT (button26),  "clicked", on_hyperlink1_activate, NULL);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button26", button26,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button26);

  tmp_toolbar_icon = create_pixmap (app1, "ginf/tb_image.xpm", TRUE);
  button27 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Image"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button27);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button27", button27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button27),  "clicked", on_image1_activate, NULL);
  gtk_widget_show (button27);

  tmp_toolbar_icon = create_pixmap (app1, "ginf/tb_table.xpm", TRUE);
  button28 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Table"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button28);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button28", button28,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_signal_connect (GTK_OBJECT (button28),  "clicked", on_new_table1_activate, NULL);

  gtk_widget_show (button28);

  tmp_toolbar_icon = create_pixmap (app1, "ginf/tb_separator.xpm", TRUE);
  button29 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Line"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button29);
        gtk_signal_connect (GTK_OBJECT (button29),  "clicked", on_line1_activate, NULL);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button29", button29,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button29);


  /* *** Add the font-toolbar. *** */

  toolbar3 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_ref (toolbar3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "toolbar3", toolbar3,
                            (GtkDestroyNotify) gtk_widget_unref);

  toolbar3 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_ref (toolbar3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "toolbar3", toolbar3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar3);
  gnome_app_add_toolbar (GNOME_APP (app1), GTK_TOOLBAR (toolbar3), "toolbar3",
                                GNOME_DOCK_ITEM_BEH_NORMAL,
                                GNOME_DOCK_TOP, 2, 0, 0);

  optionmenu2 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "optionmenu2", optionmenu2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu2);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar3), optionmenu2, NULL, NULL);
  optionmenu2_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Normal"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_normal1_activate, NULL);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 1"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_1_activate, NULL);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 2"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_2_activate, NULL);

  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 3"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_3_activate, NULL);

  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 4"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_4_activate, NULL);

  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 5"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_5_activate, NULL);

  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("Heading 6"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_heading_6_activate, NULL);

  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu2), optionmenu2_menu);

  Fonts = gtk_option_menu_new ();
  gtk_widget_ref (Fonts);
  gtk_object_set_data_full (GTK_OBJECT (app1), "Fonts", Fonts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Fonts);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar3), Fonts, NULL, NULL);
  Fonts_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Variable Width"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_font_variable_width_activate, NULL);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (Fonts_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Fixed Width"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", on_font_fixed_width_activate, NULL);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (Fonts_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (Fonts), Fonts_menu);

  spinbutton1_adj = gtk_adjustment_new (0, -2, 4, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (spinbutton1),  "changed", on_font_size_spinbutton_changed, NULL);
  gtk_widget_show (spinbutton1);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar3), spinbutton1, NULL, NULL);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton1), TRUE);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_COLORSELECTOR);
  button22 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
                                GTK_TOOLBAR_CHILD_BUTTON, NULL,
                                _("fontcolor"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button22);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button22", button22,
                                (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button22),  "clicked", on_colour1_activate, NULL);
  gtk_widget_show (button22);


  /* *** Add the formatting tool-bar. *** */

  toolbar2 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_ref (toolbar2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "toolbar2", toolbar2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar2);
  gnome_app_add_toolbar (GNOME_APP (app1), GTK_TOOLBAR (toolbar2), "toolbar2",
                                GNOME_DOCK_ITEM_BEH_NORMAL,
                                GNOME_DOCK_TOP, 3, 0, 0);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar2), GTK_RELIEF_NONE);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_BOLD);
  button12 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Bold"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button12);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button12", button12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button12),  "clicked", on_bold1_activate, NULL);
  gtk_widget_show (button12);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_ITALIC);
  button13 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Italic"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button13);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button13", button13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button13),  "clicked", on_italic1_activate, NULL);
  gtk_widget_show (button13);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_UNDERLINE);
  button14 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Underline"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button14);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button14", button14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button14),  "clicked", on_underline1_activate, NULL);
  gtk_widget_show (button14);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT);
  button33 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button33"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button33);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button33", button33,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button33),  "clicked", on_strikeout1_activate, NULL);
  gtk_widget_show (button33);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar2));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_ALIGN_LEFT);
  button15 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Left"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button15);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button15", button15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button15),  "clicked", on_left1_activate, NULL);
  gtk_widget_show (button15);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_ALIGN_CENTER);
  button16 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Center"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button16);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button16", button16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button16),  "clicked", on_centre1_activate, NULL);
  gtk_widget_show (button16);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_ALIGN_RIGHT);
  button17 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Right"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button17);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button17", button17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button17),  "clicked", on_right1_activate, NULL);
  gtk_widget_show (button17);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar2));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_INDENT);
  button18 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Indent Right"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button18);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button18", button18,
                              (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button18),  "clicked", on_increase_indent_activate, NULL);
  gtk_widget_show (button18);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_UNINDENT);
  button19 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Indent Left"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button19);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button19", button19,
                            (GtkDestroyNotify) gtk_widget_unref);
    gtk_signal_connect (GTK_OBJECT (button19),  "clicked", on_decrease_indent_activate, NULL);
  gtk_widget_show (button19);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar2));

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_BULLETED_LIST);
  button20 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Bulletted List"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button20);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button20", button20,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button20),  "clicked", on_bulletted1_activate, NULL);
  gtk_widget_show (button20);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (app1, GNOME_STOCK_PIXMAP_TEXT_NUMBERED_LIST);
  button21 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar2),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("button21"),
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (button21);
  gtk_object_set_data_full (GTK_OBJECT (app1), "button21", button21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button21),  "clicked", on_numbered1_activate, NULL);
  gtk_widget_show (button21);


  /* *** Create the note-book edit area. *** */

  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gnome_app_set_contents (GNOME_APP (app1), notebook1);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook1), FALSE);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_BOTTOM);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook1), TRUE);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport1 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "viewport1", viewport1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport1);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow4);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow4", scrolledwindow4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow4);
  gtk_container_add (GTK_CONTAINER (viewport1), scrolledwindow4);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

// begin edit widget

        html_edit1 = gtk_html_new ();
        gtk_widget_ref (html_edit1);
        gtk_object_set_data_full (GTK_OBJECT (app1), "html_edit1", html_edit1,
                            (GtkDestroyNotify) gtk_widget_unref);
        gtk_html_load_empty (GTK_HTML(html_edit1));
        gtk_html_set_editable (GTK_HTML(html_edit1), TRUE);
        gtk_widget_show (html_edit1);
        gtk_container_add (GTK_CONTAINER (scrolledwindow4), html_edit1);

// end edit widget


  label1 = gtk_label_new (_("Edit"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (label1), TRUE);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow2", scrolledwindow2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport2 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "viewport2", viewport2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), viewport2);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow5);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow5", scrolledwindow5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow5);
  gtk_container_add (GTK_CONTAINER (viewport2), scrolledwindow5);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  text1 = gtk_text_new (NULL, NULL);
  gtk_widget_ref (text1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "text1", text1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (text1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), text1);

  label2 = gtk_label_new (_("HTML"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (app1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (label2), TRUE);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow3", scrolledwindow3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow3);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  viewport3 = gtk_viewport_new (NULL, NULL);
  gtk_widget_ref (viewport3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "viewport3", viewport3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (viewport3);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), viewport3);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow6);
  gtk_object_set_data_full (GTK_OBJECT (app1), "scrolledwindow6", scrolledwindow6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow6);
  gtk_container_add (GTK_CONTAINER (viewport3), scrolledwindow6);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

// begin preview widget

  html_preview1 = gtk_html_new ();
  gtk_widget_ref (html_preview1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "html_preview1", html_preview1,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_html_load_empty (GTK_HTML (html_preview1));
  gtk_widget_show (html_preview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), html_preview1);

// end preview widget


// signal handler for switch-page

        gtk_signal_connect (GTK_OBJECT (notebook1), "switch_page", GTK_SIGNAL_FUNC(on_notebook1_switch_page), NULL);


  label3 = gtk_label_new (_("Preview"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (app1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);
  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

  appbar1 = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_ref (appbar1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "appbar1", appbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (appbar1);
  gnome_app_set_statusbar (GNOME_APP (app1), appbar1);

  gtk_signal_connect (GTK_OBJECT (app1), "destroy",
                      GTK_SIGNAL_FUNC (on_exit_activate),
                      NULL);
  gnome_app_install_menu_hints (GNOME_APP (app1), menubar1_uiinfo);
  gtk_widget_grab_focus (html_edit1);

  return app1;
}


/* ***************************************************************** */

/* *** Generic message-box creation function. *** */

GtkWidget*
create_msg_box (const gchar *msg, const gchar *title, const gchar *type)
{
  GtkWidget *messagebox1;
  GtkWidget *dialog_vbox1;
  GtkWidget *button1;
  GtkWidget *dialog_action_area1;

  messagebox1 = gnome_message_box_new (_(msg),
                              type, NULL);
  gtk_object_set_data (GTK_OBJECT (messagebox1), "messagebox1", messagebox1);
  gtk_window_set_title (GTK_WINDOW (messagebox1), _(title));
  gtk_window_set_position (GTK_WINDOW (messagebox1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (messagebox1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (messagebox1)->vbox;
  gtk_object_set_data (GTK_OBJECT (messagebox1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  gnome_dialog_append_button (GNOME_DIALOG (messagebox1), GNOME_STOCK_BUTTON_OK);
  button1 = g_list_last (GNOME_DIALOG (messagebox1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (messagebox1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  dialog_action_area1 = GNOME_DIALOG (messagebox1)->action_area;
  gtk_widget_ref (dialog_action_area1);
  gtk_object_set_data_full (GTK_OBJECT (messagebox1), "dialog_action_area1", dialog_action_area1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (gtk_widget_get_toplevel(button1)), "clicked",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy),
                      GTK_OBJECT (messagebox1));

  gtk_widget_show (messagebox1);

  GTK_WINDOW (messagebox1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (messagebox1), GTK_WIN_POS_CENTER);

  return messagebox1;
}

/* ***************************************************************** */

/* *** Edit-menu search-dialogue box. *** */

GtkWidget*
create_search_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *label1;
  GtkWidget *entry1;
  GtkWidget *frame1;
  GtkWidget *vbox2;
  GtkWidget *hbox2;
  GtkWidget *label2;
  GSList *search_pos_group_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *hbox3;
  GtkWidget *label3;
  GSList *direction_group_group = NULL;
  GtkWidget *radiobutton3;
  GtkWidget *radiobutton4;
  GtkWidget *frame2;
  GtkWidget *hbox4;
  GtkWidget *checkbutton1;
  GtkWidget *checkbutton2;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;

  extern search_properties *search_attr;
  search_attr = g_malloc (sizeof (search_properties));
  search_attr->forward = TRUE;
  search_attr->case_sensitive = FALSE;
  search_attr->regular_expression = FALSE;
  search_attr->from_beginning = TRUE;
  search_attr->done_find = FALSE;

  dialog1 = gnome_dialog_new ("Find text", NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 5);

  label1 = gtk_label_new (_("Search term:"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 5);

  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_signal_connect (GTK_OBJECT (entry1),  "changed", reset_done_find, (gpointer) OPR_FIND);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 6);

  frame1 = gtk_frame_new (_("Search Options"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, TRUE, TRUE, 0);

  label2 = gtk_label_new (_("Search from:     "));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 11);

  radiobutton1 = gtk_radio_button_new_with_label (search_pos_group_group, _("Start of file"));
  search_pos_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton1", radiobutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton1),  "clicked", search_begin_clicked, NULL);
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton1, FALSE, FALSE, 5);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton1), TRUE);

  radiobutton2 = gtk_radio_button_new_with_label (search_pos_group_group, _("Cursor position"));
  search_pos_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton2),  "clicked", search_cursor_clicked, NULL);
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton2, FALSE, FALSE, 5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, TRUE, 1);

  label3 = gtk_label_new (_("Search direction:"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox3), label3, FALSE, FALSE, 10);

  radiobutton3 = gtk_radio_button_new_with_label (direction_group_group, _("Forward     "));
  direction_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton3));
  gtk_widget_ref (radiobutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton3", radiobutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton3),  "clicked", search_fwd_clicked, NULL);
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (hbox3), radiobutton3, FALSE, FALSE, 3);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton3), TRUE);

  radiobutton4 = gtk_radio_button_new_with_label (direction_group_group, _("Backward"));
  direction_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_widget_ref (radiobutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton4", radiobutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton4),  "clicked", search_back_clicked, NULL);
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (hbox3), radiobutton4, FALSE, FALSE, 0);

  frame2 = gtk_frame_new (_("Term Options"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 5);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (frame2), hbox4);

  checkbutton1 = gtk_check_button_new_with_label (_("Case sensitive"));
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (checkbutton1),  "clicked", search_case_sens_clicked, NULL);
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (hbox4), checkbutton1, FALSE, FALSE, 10);

  checkbutton2 = gtk_check_button_new_with_label (_("Regular expression"));
  gtk_widget_ref (checkbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton2", checkbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (checkbutton2),  "clicked", search_regex_clicked, NULL);
  gtk_widget_show (checkbutton2);
  gtk_box_pack_start (GTK_BOX (hbox4), checkbutton2, FALSE, FALSE, 0);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
                                          _("Find"), GNOME_STOCK_PIXMAP_SEARCH);
  button1 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", search_find_clicked, entry1);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
                                          _("Prev"), GNOME_STOCK_PIXMAP_BACK);
  button2 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", search_next_clicked, entry1);
  gtk_widget_show (button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
                                          _("Next"), GNOME_STOCK_PIXMAP_FORWARD);
  button3 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", search_prev_clicked, entry1);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CLOSE);
  button4 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button4),  "clicked", search_close_clicked, NULL);
  gtk_widget_show (button4);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

  gtk_widget_grab_focus (button1);
  return dialog1;
}

/* ***************************************************************** */

/* *** Edit-menu replace-dialogue-box. *** */

GtkWidget*
create_replace_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *frame1;
  GtkWidget *table1;
  GtkWidget *label1;
  GtkWidget *label3;
  GtkWidget *entry1;
  GtkWidget *entry2;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;
  extern search_properties *replace_attr;
  replace_attr = g_malloc (sizeof (search_properties));
  replace_attr->forward = TRUE;
  replace_attr->case_sensitive = FALSE;
  replace_attr->regular_expression = FALSE;
  replace_attr->from_beginning = TRUE;
  replace_attr->done_find = FALSE;

  dialog1 = gnome_dialog_new (_("Replace"), NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  frame1 = gtk_frame_new (_("Find & Replace"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame1), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 10);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 20);

  label1 = gtk_label_new (_("Search term"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label3 = gtk_label_new (_("Replace term"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_signal_connect (GTK_OBJECT (entry1),  "changed", reset_done_find, (gpointer) OPR_REPLACE);
  gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry2 = gtk_entry_new ();
  gtk_widget_ref (entry2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry2);
  gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  replace_attr->w1 = entry1;
  replace_attr->w2 = entry2;

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (dialog_action_area1), 13, -1);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (dialog_action_area1), 0, -1);

  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
                                          _("Find Next"), GNOME_STOCK_PIXMAP_SEARCH);
  button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", replace_dialog_cb, (gpointer) OPR_FIND);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
                                          _("Replace"), GNOME_STOCK_PIXMAP_SRCHRPL);
  button2 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", replace_dialog_cb, (gpointer) OPR_REPLACE);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), _("Replace All"));
  button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", replace_dialog_cb, (gpointer) OPR_REPLACE_ALL);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CLOSE);
  button4 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_signal_connect (GTK_OBJECT (button4),  "clicked", replace_dialog_cb, (gpointer) OPR_CLOSE);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

  gtk_widget_grab_default (button4);
  return dialog1;
}

/* ***************************************************************** */

/* *** Edit-menu page-properties dialogue-box. *** */

GtkWidget* create_properties_dialog (int which)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *notebook1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *table1;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *label7;
  GtkWidget *label8;
  GtkWidget *label9;
  GtkWidget *colorpicker1;
  GtkWidget *colorpicker2;
  GtkWidget *colorpicker3;
  GtkWidget *colorpicker4;
  GtkWidget *colorpicker5;
  GtkWidget *checkbutton1;
  GtkWidget *frame2;
  GtkWidget *table2;
  GtkWidget *checkbutton2;
  GtkWidget *button4;
  GtkWidget *label10;
  GtkWidget *entry1;
  GtkWidget *hseparator1;
  GtkWidget *label4;
  GtkWidget *label1;
  GtkWidget *vbox2;
  GtkWidget *frame3;
  GtkWidget *table4;
  GtkWidget *label11;
  GtkWidget *label12;
  GtkWidget *label13;
  GtkWidget *entry2;
  GtkWidget *entry3;
  GtkWidget *entry4;
  GtkWidget *frame4;
  GtkWidget *table5;
  GtkWidget *label14;
  GtkWidget *label15;
  GtkWidget *entry5;
  GtkWidget *entry6;
  GtkWidget *label16;
  GtkWidget *label2;
  GtkWidget *frame5;
  GtkWidget *table6;
  GtkWidget *label17;
  GtkWidget *label18;
  GtkWidget *entry7;
  GtkWidget *entry8;
  GtkWidget *label19;
  GtkWidget *hbuttonbox1;
  GtkWidget *button6;
  GtkWidget *button7;
  GtkWidget *hseparator2;
  GtkWidget *label3;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  extern ginf_properties DOC_PROPERTIES;
  guint red, green, blue;
  extern widgets *meta_widget_set;
  extern widgets *properties_widget_set;
  meta_widget_set = g_malloc (sizeof (widgets));
  properties_widget_set = g_malloc (sizeof (widgets));

  dialog1 = gnome_dialog_new ("Properties", NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);


  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), notebook1, TRUE, TRUE, 0);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 5);

  frame1 = gtk_frame_new (_("Document Colors"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);

  table1 = gtk_table_new (6, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);


 gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame1), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 1);

  label5 = gtk_label_new (_("Normal Text"));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label5, 200, -2);

  label6 = gtk_label_new (_("Hyperlink"));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_table_attach (GTK_TABLE (table1), label6, 0, 1, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label7 = gtk_label_new (_("Selected Link"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table1), label7, 0, 1, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label8 = gtk_label_new (_("Visited Link"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_table_attach (GTK_TABLE (table1), label8, 0, 1, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label9 = gtk_label_new (_("Background"));
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_table_attach (GTK_TABLE (table1), label9, 0, 1, 5, 6,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  colorpicker1 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker1", colorpicker1,
                            (GtkDestroyNotify) gtk_widget_unref);

  red = (DOC_PROPERTIES.text & 0xbff0000) >> 16;
  green = (DOC_PROPERTIES.text & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.text & 0x0000ff;

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker1), red, green, blue, 0);

  gtk_widget_show (colorpicker1);
  gtk_table_attach (GTK_TABLE (table1), colorpicker1, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  colorpicker2 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker2", colorpicker2,
                            (GtkDestroyNotify) gtk_widget_unref);

  red = (DOC_PROPERTIES.link & 0xff0000) >> 16;
  green = (DOC_PROPERTIES.link & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.link & 0x0000ff;

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker2), red, green, blue, 0);
  gtk_widget_show (colorpicker2);

  gtk_table_attach (GTK_TABLE (table1), colorpicker2, 1, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);


  colorpicker3 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker3", colorpicker3,
                            (GtkDestroyNotify) gtk_widget_unref);

  red = (DOC_PROPERTIES.alink & 0xff0000) >> 16;
  green = (DOC_PROPERTIES.alink & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.alink & 0x0000ff;

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker3), red, green, blue, 0);

  gtk_widget_show (colorpicker3);
  gtk_table_attach (GTK_TABLE (table1), colorpicker3, 1, 2, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  colorpicker4 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker4", colorpicker4,
                            (GtkDestroyNotify) gtk_widget_unref);
  red = (DOC_PROPERTIES.vlink & 0xff0000) >> 16;
  green = (DOC_PROPERTIES.vlink & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.vlink & 0x0000ff;

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker4), red, green, blue, 0);

  gtk_widget_show (colorpicker4);
  gtk_table_attach (GTK_TABLE (table1), colorpicker4, 1, 2, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  colorpicker5 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker5", colorpicker5,
                        (GtkDestroyNotify) gtk_widget_unref);
  red = (DOC_PROPERTIES.bgcolor & 0xff0000) >> 16;
  green = (DOC_PROPERTIES.bgcolor & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.bgcolor & 0x0000ff;
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker5), red, green, blue, 0);

  gtk_widget_show (colorpicker5);
  gtk_table_attach (GTK_TABLE (table1), colorpicker5, 1, 2, 5, 6,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  checkbutton1 = gtk_check_button_new_with_label (_("Use Custom Colors"));
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton1);
  gtk_table_attach (GTK_TABLE (table1), checkbutton1, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  frame2 = gtk_frame_new (_("Background Image"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);

  table2 = gtk_table_new (2, 3, FALSE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (frame2), table2);
  gtk_container_set_border_width (GTK_CONTAINER (table2), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table2), 5);

  checkbutton2 = gtk_check_button_new_with_label (_("Use Background Image"));
  gtk_widget_ref (checkbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton2", checkbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton2);
  gtk_table_attach (GTK_TABLE (table2), checkbutton2, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  button4 = gtk_button_new_with_label (_("Browse..."));
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_table_attach (GTK_TABLE (table2), button4, 2, 3, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label10 = gtk_label_new (_("Image Overrides Background Color.\nImage can be seen only in a browser."));
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_table_attach (GTK_TABLE (table2), label10, 0, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_table_attach (GTK_TABLE (table2), entry1, 1, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_signal_connect (GTK_OBJECT (button4),  "clicked", on_dibox_browse_activate, (gpointer) entry1);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hseparator1", hseparator1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 3);

  label4 = gtk_label_new (_("Changes will be applied to the current document."));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox1), label4, FALSE, FALSE, 3);

  label1 = gtk_label_new (_("Colors and Background"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox2);

  frame3 = gtk_frame_new (_("Page Attributes"));
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox2), frame3, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 5);

  table4 = gtk_table_new (3, 2, FALSE);
  gtk_widget_ref (table4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table4", table4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (frame3), table4);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table4), 10);

  label11 = gtk_label_new (_("Title"));
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table4), label11, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label11, 100, -2);

  label12 = gtk_label_new (_("Author"));
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table4), label12, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

label13 = gtk_label_new (_("Description"));
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_table_attach (GTK_TABLE (table4), label13, 0, 1, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  entry2 = gtk_entry_new ();
  gtk_widget_ref (entry2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (entry2), DOC_PROPERTIES.title);
  gtk_widget_show (entry2);
  gtk_table_attach (GTK_TABLE (table4), entry2, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry3 = gtk_entry_new ();
  gtk_widget_ref (entry3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry3", entry3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (entry3), DOC_PROPERTIES.author);
  gtk_widget_show (entry3);
  gtk_table_attach (GTK_TABLE (table4), entry3, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry4 = gtk_entry_new ();
  gtk_widget_ref (entry4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry4", entry4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (entry4), DOC_PROPERTIES.description);
  gtk_widget_show (entry4);
  gtk_table_attach (GTK_TABLE (table4), entry4, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  frame4 = gtk_frame_new (_("More Attributes"));
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox2), frame4, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 5);

  table5 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table5", table5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table5);
  gtk_container_add (GTK_CONTAINER (frame4), table5);
  gtk_container_set_border_width (GTK_CONTAINER (table5), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table5), 10);

  label14 = gtk_label_new (_("Keywords"));
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_table_attach (GTK_TABLE (table5), label14, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label14, 100, -2);

  label15 = gtk_label_new (_("Classification"));
  gtk_widget_ref (label15);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label15", label15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label15);
  gtk_table_attach (GTK_TABLE (table5), label15, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  entry5 = gtk_entry_new ();
  gtk_widget_ref (entry5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry5", entry5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (entry5), DOC_PROPERTIES.keywords);
  gtk_widget_show (entry5);
  gtk_table_attach (GTK_TABLE (table5), entry5, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry6 = gtk_entry_new ();
  gtk_widget_ref (entry6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry6", entry6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_entry_set_text (GTK_ENTRY (entry6), DOC_PROPERTIES.classification);
  gtk_widget_show (entry6);
  gtk_table_attach (GTK_TABLE (table5), entry6, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label16 = gtk_label_new (_("For other meta tags, click on advanced."));
  gtk_widget_ref (label16);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label16", label16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label16);
  gtk_box_pack_start (GTK_BOX (vbox2), label16, FALSE, FALSE, 0);
  gtk_widget_set_usize (label16, -2, 65);

  label2 = gtk_label_new (_("Page Properties"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  frame5 = gtk_frame_new (_("Meta Tags"));
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame5);
  gtk_container_add (GTK_CONTAINER (notebook1), frame5);
  gtk_container_set_border_width (GTK_CONTAINER (frame5), 5);

  table6 = gtk_table_new (5, 2, FALSE);
  gtk_widget_ref (table6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table6", table6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table6);
  gtk_container_add (GTK_CONTAINER (frame5), table6);
  gtk_container_set_border_width (GTK_CONTAINER (table6), 10);
  gtk_table_set_row_spacings (GTK_TABLE (table6), 20);

  label17 = gtk_label_new (_("Name"));
  gtk_widget_ref (label17);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label17", label17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label17);
  gtk_table_attach (GTK_TABLE (table6), label17, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (label17, 100, -2);

  label18 = gtk_label_new (_("Content"));
  gtk_widget_ref (label18);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label18", label18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label18);
  gtk_table_attach (GTK_TABLE (table6), label18, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  entry7 = gtk_entry_new ();
  gtk_widget_ref (entry7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry7", entry7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry7);
  gtk_table_attach (GTK_TABLE (table6), entry7, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry8 = gtk_entry_new ();
  gtk_widget_ref (entry8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry8", entry8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry8);
  gtk_table_attach (GTK_TABLE (table6), entry8, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label19 = gtk_label_new ("View the meta tags at the source view.");
  gtk_widget_ref (label19);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label19", label19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table6), label19, 0, 2, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);


  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_table_attach (GTK_TABLE (table6), hbuttonbox1, 0, 2, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox1), 0, -1);

  button6 = gtk_button_new_with_label (_("Add"));
  gtk_widget_ref (button6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button6", button6,
                            (GtkDestroyNotify) gtk_widget_unref);

  meta_widget_set->w1 = entry7;
  meta_widget_set->w2 = entry8;

  gtk_signal_connect (GTK_OBJECT (button6),  "clicked", on_meta_tags_add_activate, NULL);
  gtk_widget_show (button6);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button6);

  GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);

  button7 = gtk_button_new_with_label (_("Clear"));
  gtk_widget_ref (button7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button7", button7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button7);
  gtk_signal_connect (GTK_OBJECT (button7),  "clicked", on_meta_tags_clear_activate, NULL);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button7);

  GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_ref (hseparator2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hseparator2", hseparator2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator2);
  gtk_table_attach (GTK_TABLE (table6), hseparator2, 0, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label3 = gtk_label_new (_("Advanced"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

/*
checkbutton1 = usercustom colors
colorpicker1-5 = text, link, alink, vlink, bgcolor
checkbutton2 = user bg image
entry1 = bgimage
entry2-4 title, author, desc
entry 5-6 keywords, classification
*/
  properties_widget_set->w1 = checkbutton1;
  properties_widget_set->w2 = colorpicker1;
  properties_widget_set->w3 = colorpicker2;
  properties_widget_set->w4 = colorpicker3;
  properties_widget_set->w5 = colorpicker4;
  properties_widget_set->w6 = colorpicker5;
  properties_widget_set->w7 = checkbutton2;
  properties_widget_set->w8 = entry1;
  properties_widget_set->w9 = entry2;
  properties_widget_set->w10 = entry3;
  properties_widget_set->w11 = entry4;
  properties_widget_set->w12 = entry5;
  properties_widget_set->w13 = entry6;

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", on_properties_ok_activate, (gpointer) OPR_OK);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", on_properties_ok_activate, (gpointer) OPR_APPLY);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", on_properties_cancel_activate, NULL);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  gtk_notebook_set_page (GTK_NOTEBOOK (notebook1), which);

  return dialog1;

}

/* ***************************************************************** */

/* *** Insert-menu add-hyperlink dialogue-box. *** */

GtkWidget*
create_link_dialog (void)
{
  extern GtkWidget *html_edit1;
  GtkWidget *glade_menuitem;
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *vbox2;
  GtkWidget *label1;
  GtkWidget *entry1;
  GtkWidget *frame2;
  GtkWidget *table1;
  GtkWidget *button4;
  GtkWidget *label2;
  GtkWidget *entry2;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *optionmenu1;
  GtkWidget *optionmenu1_menu;
  extern link_properties *link_props;
  link_props = g_malloc (sizeof (link_properties));

  link_props->copied_text = NULL;
  link_props->type = 0;


  dialog1 = gnome_dialog_new (_("Insert hyperlink"), NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  frame1 = gtk_frame_new (_("Link Source"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);

  vbox2 = gtk_vbox_new (FALSE, 5);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 5);

  label1 = gtk_label_new (_("Enter the text of the link"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  link_props->copied_text = html_engine_get_selection_string (GTK_HTML(html_edit1)->engine);
  if (link_props->copied_text != NULL) {
          gtk_entry_set_text (GTK_ENTRY (entry1), link_props->copied_text);
  }
  gtk_box_pack_start (GTK_BOX (vbox2), entry1, FALSE, FALSE, 0);

  frame2 = gtk_frame_new (_("Link Target"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);

  table1 = gtk_table_new (2, 3, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame2), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

  button4 = gtk_button_new_with_label (_("Browse ..."));
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_table_attach (GTK_TABLE (table1), button4, 2, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label2 = gtk_label_new (_("Enter link to page location or local file"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table1), label2, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  optionmenu1 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu1", optionmenu1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu1);
  gtk_table_attach (GTK_TABLE (table1), optionmenu1, 0, 1, 1, 2,
                    (GtkAttachOptions) ( GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_usize (optionmenu1, 45, -2);
  optionmenu1_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("http://"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)0);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("https://"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)1);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("ftp://"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)2);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("gopher://"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)3);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("telnet://"));
  gtk_widget_show (glade_menuitem);
    gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)4);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("mailo:"));
  gtk_widget_show (glade_menuitem);
    gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)5);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("news:"));
  gtk_widget_show (glade_menuitem);
    gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)6);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label (_("none"));
  gtk_widget_show (glade_menuitem);
    gtk_signal_connect (GTK_OBJECT (glade_menuitem),  "activate", link_opts_activate, (gpointer)7);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);

  entry2 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry2);
  gtk_table_attach (GTK_TABLE (table1), entry2, 1, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_signal_connect (GTK_OBJECT (button4),  "clicked", on_dibox_browse_activate, entry2);

  link_props->w1 = entry1;
  link_props->w2 = entry2;
  link_props->w3 = optionmenu1;
  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", insert_link_cb, (gpointer) OPR_OK);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", insert_link_cb, (gpointer) OPR_APPLY);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", insert_link_cb, (gpointer) OPR_CANCEL);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  return dialog1;
}

/* ***************************************************************** */

/* *** Insert-menu insert-image dialogue-box. *** */

GtkWidget*
create_image_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *hbox1;
  GtkWidget *label1;
  GtkWidget *entry1;
  GtkWidget *button4;
  GtkWidget *frame3;
  GtkWidget *vbox2;
  GtkWidget *hbox4;
  GtkWidget *label2;
  GtkObject *spinbutton1_adj;
  GtkWidget *spinbutton1;
  GtkWidget *label3;
  GtkWidget *hbox5;
  GtkWidget *label4;
  GtkObject *spinbutton2_adj;
  GtkWidget *spinbutton2;
  GtkWidget *label5;
  GtkWidget *frame4;
  GtkWidget *vbox3;
  GtkWidget *hbox6;
  GtkWidget *label6;
  GtkWidget *entry2;
  GtkWidget *hbox7;
  GtkWidget *checkbutton1;
  GtkObject *spinbutton3_adj;
  GtkWidget *spinbutton3;
  GtkWidget *label7;
  GtkWidget *hbox8;
  GtkWidget *checkbutton2;
  GtkWidget *hbox9;
  GtkWidget *label8;
  GtkObject *spinbutton4_adj;
  GtkWidget *spinbutton4;
  GSList *height_group = NULL;
  GtkWidget *radiobutton4;
  GtkWidget *radiobutton5;
  GtkWidget *hbox10;
  GtkWidget *label9;
  GtkObject *spinbutton5_adj;
  GtkWidget *spinbutton5;
  GSList *width_group = NULL;
  GtkWidget *radiobutton6;
  GtkWidget *radiobutton7;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;

  extern widgets* img_widget_set;
  extern image_properties* img_props_set;
  img_widget_set = g_malloc (sizeof (widgets));
  img_props_set = g_malloc (sizeof (image_properties));

  img_props_set->align = 0;
  img_props_set->has_border = FALSE;
  img_props_set->has_cust_size = FALSE;
  img_props_set->width_in_percent = FALSE;
  img_props_set->height_in_percent = FALSE;



  dialog1 = gnome_dialog_new ("Insert Image", NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  frame1 = gtk_frame_new (_("Image"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 5);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (frame1), hbox1);

  label1 = gtk_label_new (_("Location:"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label1), 5, 3);

  entry1 = gtk_entry_new_with_max_length (256);
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 5);

  button4 = gtk_button_new_with_label (_(" Browse.. "));
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked", table_bgimage_browse_clicked, entry1);
  gtk_widget_show (button4);
  gtk_box_pack_start (GTK_BOX (hbox1), button4, FALSE, FALSE, 3);
  gtk_button_set_relief (GTK_BUTTON (button4), GTK_RELIEF_HALF);

  frame3 = gtk_frame_new (_("Spacing"));
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame3), vbox2);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 3);

  label2 = gtk_label_new (_("Left and right:          "));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox4), label2, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label2), 5, 3);

  spinbutton1_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox4), spinbutton1, FALSE, FALSE, 0);

  label3 = gtk_label_new (_("pixels"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox4), label3, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label3), 5, 0);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox5, TRUE, TRUE, 3);

  label4 = gtk_label_new (_("Top and bottom:       "));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox5), label4, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label4), 5, 3);

  spinbutton2_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox5), spinbutton2, FALSE, FALSE, 0);

  label5 = gtk_label_new (_("pixels"));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox5), label5, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label5), 5, 3);

  frame4 = gtk_frame_new (_("Optional Attributes"));
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox1), frame4, TRUE, TRUE, 5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame4), vbox3);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox6, TRUE, TRUE, 5);

  label6 = gtk_label_new (_("Alternate text:       "));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox6), label6, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label6), 5, 3);

  entry2 = gtk_entry_new_with_max_length (256);
  gtk_widget_ref (entry2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox6), entry2, TRUE, TRUE, 5);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox7, TRUE, TRUE, 5);

  checkbutton1 = gtk_check_button_new_with_label (_("Border present"));
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT(checkbutton1), "clicked", img_border_toggle, NULL);
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (hbox7), checkbutton1, FALSE, FALSE, 4);

  spinbutton3_adj = gtk_adjustment_new (1, 1, 100, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_ref (spinbutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton3", spinbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton3);
  gtk_box_pack_start (GTK_BOX (hbox7), spinbutton3, FALSE, FALSE, 5);

  label7 = gtk_label_new (_("pixels"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox7), label7, FALSE, FALSE, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox8", hbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox8, FALSE, FALSE, 0);

  checkbutton2 = gtk_check_button_new_with_label (_("Custom size"));
  gtk_widget_ref (checkbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton2", checkbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT(checkbutton2), "clicked", img_custsize_toggle, NULL);
  gtk_widget_show (checkbutton2);
  gtk_box_pack_start (GTK_BOX (hbox8), checkbutton2, FALSE, FALSE, 4);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox9, TRUE, TRUE, 3);

  label8 = gtk_label_new (_("Height:"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox9), label8, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label8), 30, 0);

  spinbutton4_adj = gtk_adjustment_new (1, 1, 1000, 1, 10, 10);
  spinbutton4 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton4_adj), 1, 0);
  gtk_widget_ref (spinbutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton4", spinbutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton4);
  gtk_box_pack_start (GTK_BOX (hbox9), spinbutton4, FALSE, FALSE, 2);

  radiobutton4 = gtk_radio_button_new_with_label (height_group, _("pixels"));
  height_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_widget_ref (radiobutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton4", radiobutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (hbox9), radiobutton4, FALSE, FALSE, 5);

  radiobutton5 = gtk_radio_button_new_with_label (height_group, _("%"));
  height_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton5));
  gtk_widget_ref (radiobutton5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton5", radiobutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton5);
  gtk_box_pack_start (GTK_BOX (hbox9), radiobutton5, FALSE, FALSE, 5);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox10, TRUE, TRUE, 3);

  label9 = gtk_label_new (_("Width:"));
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox10), label9, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label9), 30, 0);

  spinbutton5_adj = gtk_adjustment_new (1, 1, 1000, 1, 10, 10);
  spinbutton5 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton5_adj), 1, 0);
  gtk_widget_ref (spinbutton5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton5", spinbutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton5);
  gtk_box_pack_start (GTK_BOX (hbox10), spinbutton5, FALSE, FALSE, 5);

  radiobutton6 = gtk_radio_button_new_with_label (width_group, _("pixels"));
  width_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton6));
  gtk_widget_ref (radiobutton6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton6", radiobutton6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton6);
  gtk_box_pack_start (GTK_BOX (hbox10), radiobutton6, FALSE, FALSE, 5);

  radiobutton7 = gtk_radio_button_new_with_label (width_group, _("%"));
  width_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton7));
  gtk_widget_ref (radiobutton7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton7", radiobutton7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton7);
  gtk_box_pack_start (GTK_BOX (hbox10), radiobutton7, FALSE, FALSE, 5);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", on_image_ok_activate, NULL);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", on_image_apply_activate, NULL);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", on_image_cancel_activate, NULL);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  img_widget_set->w1 = entry1;       // Img. location
  img_widget_set->w2 = spinbutton1;  // Horiz. spacing
  img_widget_set->w3 = spinbutton2;  // Vert. spacing
  img_widget_set->w4 = entry2;       // Alt text
  img_widget_set->w5 = spinbutton3;  // Border
  img_widget_set->w6 = spinbutton4;  // Height
  img_widget_set->w7 = spinbutton5;  // Width

  return dialog1;
}

/* ***************************************************************** */

/* *** Insert-menu HTML-insert dialogue-box. *** */

GtkWidget*
create_html_insert_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *scrolledwindow1;
  GtkWidget *html_text;
  GtkWidget *label1;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;

  dialog1 = gnome_dialog_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);


  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  html_text = gtk_text_new (NULL, NULL);
  gtk_widget_ref (html_text);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "html_text", html_text,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (html_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), html_text);
  gtk_text_set_editable (GTK_TEXT (html_text), TRUE);

  label1 = gtk_label_new (_("Any scripts entered will not be checked for syntax."));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 10);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_html_ok),
                      html_text);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (on_html_apply),
                      html_text);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (on_html_cancel),
                      NULL);

  return dialog1;
}

/* ***************************************************************** */

/* *** Format-menu font-colour dialogue-box. *** */

GtkWidget*
create_font_color_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox1;
  GtkWidget *label1;
  GtkWidget *colorpicker1;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button3;

  extern ginf_properties  DOC_PROPERTIES;

  guint8 red, blue, green;

  dialog1 = gnome_dialog_new ("Pick Color", NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);


  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new (_("Pick Font Color"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);

  colorpicker1 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker1", colorpicker1,
                            (GtkDestroyNotify) gtk_widget_unref);

  red = (DOC_PROPERTIES.text & 0xff0000) >> 16;
  green = (DOC_PROPERTIES.text & 0x00ff00) >> 8 ;
  blue = DOC_PROPERTIES.text & 0x0000ff;

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker1), red, green, blue, 0);
  gtk_widget_show (colorpicker1);
  gtk_box_pack_start (GTK_BOX (hbox1), colorpicker1, FALSE, FALSE, 0);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", on_font_color_ok_activate, (gpointer) colorpicker1);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", on_font_color_cancel_activate, NULL);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  return dialog1;
}

/* ***************************************************************** */

/* *** Table-menu insert-table dialogue-box. *** */

GtkWidget*
create_table_dialog (void)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *hbox1;
  GtkWidget *label1;
  GtkObject *spinbutton1_adj;
  GtkWidget *spinbutton1;
  GtkWidget *label10;
  GtkWidget *label2;
  GtkObject *spinbutton2_adj;
  GtkWidget *spinbutton2;
  GtkWidget *frame2;
  GtkWidget *hbox2;
  GSList *align_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *radiobutton3;
  GtkWidget *frame3;
  GtkWidget *vbox2;
  GtkWidget *hbox3;
  GtkWidget *checkbutton1;
  GtkObject *spinbutton3_adj;
  GtkWidget *spinbutton3;
  GtkWidget *label3;
  GtkWidget *hbox4;
  GtkWidget *label4;
  GtkObject *spinbutton4_adj;
  GtkWidget *spinbutton4;
  GtkWidget *label5;
  GtkWidget *hbox5;
  GtkWidget *label6;
  GtkObject *spinbutton5_adj;
  GtkWidget *spinbutton5;
  GtkWidget *label7;
  GtkWidget *hbox6;
  GtkWidget *label8;
  GtkObject *spinbutton6_adj;
  GtkWidget *spinbutton6;
  GSList *width_group = NULL;
  GtkWidget *radiobutton4;
  GtkWidget *radiobutton5;
  GtkWidget *frame5;
  GtkWidget *vbox3;
  GtkWidget *checkbutton3;
  GtkWidget *hbox9;
  GSList *background_group = NULL;
  GtkWidget *radiobutton8;
  GtkWidget *colorpicker1;
  GtkWidget *hbox10;
  GtkWidget *radiobutton9;
  GtkWidget *entry2;
  GtkWidget *button5;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;

  extern widgets *tbl_widget_set;
  extern table_properties *tbl_props_set;

  tbl_widget_set = g_malloc (sizeof (widgets));
  tbl_props_set = g_malloc (sizeof (table_properties));

  tbl_props_set->has_border = FALSE;
  tbl_props_set->has_bgnd = FALSE;
  tbl_props_set->bgnd_is_clr = TRUE;
  tbl_props_set->align = 0;
  tbl_props_set->width_in_percent = TRUE;
  tbl_props_set->height_in_percent = TRUE;

  dialog1 = gnome_dialog_new (_("Insert Table"), NULL);
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);


  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 5);

  frame1 = gtk_frame_new (_("Table"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (frame1), hbox1);

  label1 = gtk_label_new (_("Number of rows:"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label1), 10, 4);

  spinbutton1_adj = gtk_adjustment_new (1, 1, 100, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton1, FALSE, TRUE, 0);

  label10 = gtk_label_new ("");
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox1), label10, TRUE, FALSE, 0);

  label2 = gtk_label_new (_("Number of columns:"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox1), label2, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label2), 10, 4);

  spinbutton2_adj = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton2, FALSE, TRUE, 0);

  frame2 = gtk_frame_new (_("Alignment"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 10);

  hbox2 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (frame2), hbox2);

  radiobutton1 = gtk_radio_button_new_with_label (align_group, _("Left"));
  align_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton1", radiobutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton1), "clicked", table_left_align_clicked, NULL);
  gtk_widget_show (radiobutton1);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton1, FALSE, FALSE, 0);

  radiobutton2 = gtk_radio_button_new_with_label (align_group, _("Centre"));
  align_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton2), "clicked", table_center_align_clicked, NULL);
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton2, FALSE, FALSE, 0);


  radiobutton3 = gtk_radio_button_new_with_label (align_group, _("Right"));
  align_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton3));
  gtk_widget_ref (radiobutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton3", radiobutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton3), "clicked", table_right_align_clicked, NULL);
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (hbox2), radiobutton3, FALSE, FALSE, 0);

  frame3 = gtk_frame_new (_("Attributes"));
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame3), vbox2);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox3, TRUE, TRUE, 5);

  checkbutton1 = gtk_check_button_new_with_label (_("Border line present    "));
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT(checkbutton1), "clicked", table_border_toggle, NULL);
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (hbox3), checkbutton1, FALSE, FALSE, 5);

  spinbutton3_adj = gtk_adjustment_new (1, 1, 100, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_ref (spinbutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton3", spinbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton3);
  gtk_box_pack_start (GTK_BOX (hbox3), spinbutton3, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton3), TRUE);

  label3 = gtk_label_new (_("pixels wide"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox3), label3, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label3), 5, 0);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 5);

  label4 = gtk_label_new (_("Cell spacing                   "));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox4), label4, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label4), 6, 0);

  spinbutton4_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton4 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton4_adj), 1, 0);
  gtk_widget_ref (spinbutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton4", spinbutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton4);
  gtk_box_pack_start (GTK_BOX (hbox4), spinbutton4, FALSE, FALSE, 0);

  label5 = gtk_label_new (_("pixels between cells"));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox4), label5, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label5), 5, 0);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox5, TRUE, TRUE, 5);

  label6 = gtk_label_new (_("Cell padding                   "));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox5), label6, FALSE, FALSE, 5);

  spinbutton5_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton5 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton5_adj), 1, 0);
  gtk_widget_ref (spinbutton5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton5", spinbutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton5);
  gtk_box_pack_start (GTK_BOX (hbox5), spinbutton5, FALSE, FALSE, 1);

  label7 = gtk_label_new (_("pixels within cells"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox5), label7, FALSE, FALSE, 5);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox6, TRUE, TRUE, 5);

  label8 = gtk_label_new (_("Table width                   "));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox6), label8, FALSE, FALSE, 5);

  spinbutton6_adj = gtk_adjustment_new (100, 1, 100, 1, 10, 10);
  spinbutton6 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton6_adj), 1, 0);
  gtk_widget_ref (spinbutton6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton6", spinbutton6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton6);
  gtk_box_pack_start (GTK_BOX (hbox6), spinbutton6, FALSE, FALSE, 7);

  radiobutton4 = gtk_radio_button_new_with_label (width_group, _("% of window"));
  width_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_widget_ref (radiobutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton4", radiobutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton4), "clicked", table_width_percent_clicked, spinbutton6);
  gtk_widget_show (radiobutton4);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobutton4, FALSE, FALSE, 0);

  radiobutton5 = gtk_radio_button_new_with_label (width_group, _("pixels"));
  width_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton5));
  gtk_widget_ref (radiobutton5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton5", radiobutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton5), "clicked", table_width_pixel_clicked, spinbutton6);
  gtk_widget_show (radiobutton5);
  gtk_box_pack_start (GTK_BOX (hbox6), radiobutton5, FALSE, FALSE, 0);

  frame5 = gtk_frame_new (_("Background"));
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox1), frame5, TRUE, TRUE, 5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (frame5), vbox3);

  checkbutton3 = gtk_check_button_new_with_label (_("Enable background"));
  gtk_widget_ref (checkbutton3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton3", checkbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT(checkbutton3), "clicked", table_bgnd_toggle, NULL);
  gtk_widget_show (checkbutton3);
  gtk_box_pack_start (GTK_BOX (vbox3), checkbutton3, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (checkbutton3), 5);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox9, TRUE, TRUE, 5);

  radiobutton8 = gtk_radio_button_new_with_label (background_group, _("Use colour"));
  background_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton8));
  gtk_widget_ref (radiobutton8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton8", radiobutton8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton8), "clicked", table_bgcolor_clicked, NULL);
  gtk_widget_show (radiobutton8);
  gtk_box_pack_start (GTK_BOX (hbox9), radiobutton8, FALSE, FALSE, 5);

  colorpicker1 = gnome_color_picker_new ();
  gtk_widget_ref (colorpicker1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "colorpicker1", colorpicker1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (colorpicker1);
  gtk_box_pack_start (GTK_BOX (hbox9), colorpicker1, FALSE, FALSE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (vbox3), hbox10, TRUE, TRUE, 5);

  radiobutton9 = gtk_radio_button_new_with_label (background_group, _("Use image"));
  background_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton9));
  gtk_widget_ref (radiobutton9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "radiobutton9", radiobutton9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (radiobutton8), "clicked", table_bgimage_clicked, NULL);
  gtk_widget_show (radiobutton9);
  gtk_box_pack_start (GTK_BOX (hbox10), radiobutton9, FALSE, FALSE, 5);

  entry2 = gtk_entry_new_with_max_length (256);
  gtk_widget_ref (entry2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry2", entry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (hbox10), entry2, FALSE, FALSE, 0);

  button5 = gtk_button_new_with_label (_("   Browse..   "));
  gtk_widget_ref (button5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button5", button5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked", table_bgimage_browse_clicked, entry2);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox10), button5, FALSE, FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (button5), 1);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button1),  "clicked", on_table_ok_activate, NULL);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button2),  "clicked", on_table_apply_activate, NULL);
  gtk_widget_show (button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = GTK_WIDGET (g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (button3),  "clicked", on_table_cancel_activate, NULL);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);


  tbl_widget_set->w1 = spinbutton1;  // Rows
  tbl_widget_set->w2 = spinbutton2;  // Columns
  tbl_widget_set->w3 = spinbutton3;  // Border-line width
  tbl_widget_set->w4 = spinbutton4;  // Spacing
  tbl_widget_set->w5 = spinbutton5;  // Padding
  tbl_widget_set->w6 = spinbutton6;  // Table width
  tbl_widget_set->w7 = entry2;       // Background-image
  tbl_widget_set->w8 = colorpicker1; // GNOME colour-picker.

  return dialog1;
}

/* ***************************************************************** */

/* *** Ginf About dialogue-box. *** */

GtkWidget* create_about1 (void)
{
  const gchar *authors[] = {
    "Krishna Jaidev S <jaidev@symonds.net>",
    "N Deepak <deepak@phreaker.net>",
    "Monish M <monish@hotpop.com>",
    "Nandakumar K <nandakumars_10@yahoo.com>",
    NULL
  };
  GtkWidget *about1;

  about1 = gnome_about_new ("Ginf", VERSION,
                        _("Ginf Is Not FrontPage"),
                        authors,
                        _("Ginf stands for 'Ginf is not FrontPage'. It was coded as a mini-project in the curriculum of computer science and engineering in the sixth semester bachelor of engineering (BE) course, Karnataka, India."),
                        NULL);
  gtk_object_set_data (GTK_OBJECT (about1), "about1", about1);
  gtk_window_set_modal (GTK_WINDOW (about1), TRUE);
  GTK_WINDOW (about1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW (about1), GTK_WIN_POS_CENTER);


  return about1;
}

